######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    lookup_prep char(1), # Have the lookups been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=field
    tab_pressed smallint # (boolean) was the tab key pressed?


######################################################################
function llh_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    # Trap fatal errors
    whenever error call error_handler

    # Initialize some variables
    let exit_level = 0
    let tab_pressed = false


    input p_xcntrc.* without defaults from s_cntrc.*

      # All entry fields must have before and after field processing
      before field curr_asset call llh_b_field("stxcntrc.curr_asset")
        goto next_field
      before field fixed_asset call llh_b_field("stxcntrc.fixed_asset")
        goto next_field
      before field curr_liab call llh_b_field("stxcntrc.curr_liab")
        goto next_field
      before field long_term_liab call llh_b_field("stxcntrc.long_term_liab")
        goto next_field
      before field capital call llh_b_field("stxcntrc.capital")
        goto next_field
      before field income call llh_b_field("stxcntrc.income")
        goto next_field
      before field cost_goods call llh_b_field("stxcntrc.cost_goods")
        goto next_field
      before field expense call llh_b_field("stxcntrc.expense")
        goto next_field

      after field curr_asset call llh_a_field()
        goto next_field
      after field fixed_asset call llh_a_field()
        goto next_field
      after field curr_liab call llh_a_field()
        goto next_field
      after field long_term_liab call llh_a_field()
        goto next_field
      after field capital call llh_a_field()
        goto next_field
      after field income call llh_a_field()
        goto next_field
      after field cost_goods call llh_a_field()
        goto next_field
      after field expense call llh_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 1 then call llh_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after input' logic before exiting input
        call llh_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_key("llh_input")  # Map the key to the event
        call llh_event()

      # Programmatic next field logic
      label next_field:
        let scratch = nxt_fld
        let nxt_fld = null
        case
          when scratch is null  # No need to go through 'case'
          when scratch = "curr_asset" next field curr_asset
          when scratch = "fixed_asset" next field fixed_asset
          when scratch = "curr_liab" next field curr_liab
          when scratch = "long_term_liab" next field long_term_liab
          when scratch = "capital" next field capital
          when scratch = "income" next field income
          when scratch = "cost_goods" next field cost_goods
          when scratch = "expense" next field expense
          when scratch = "exit input" goto end_input
        end case
    end input

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# llh_input()

######################################################################
function llh_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field level exit
    let exit_level = 1

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# llh_b_field()

######################################################################
function llh_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # No more exit levels required
    let exit_level = 0

    # After field logic
    case
      when scr_fld = "curr_asset"
        if p_xcntrc.curr_asset < 1000 or
           p_xcntrc.curr_asset > 9999 or
           p_xcntrc.curr_asset is null
        then
            let p_xcntrc.curr_asset = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "curr_asset"
            return
        end if
        if cash_warning(p_xcntrc.curr_asset)
        then
            call fg_err(2) # Checking Accounts Out Of Range
            let nxt_fld = "curr_asset"
            return
        end if
      when scr_fld = "fixed_asset"
        if p_xcntrc.fixed_asset < 1000 or
           p_xcntrc.fixed_asset > 9999 or
           p_xcntrc.fixed_asset is null
        then
            let p_xcntrc.fixed_asset = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "fixed_asset"
            return
        end if
      when scr_fld = "curr_liab"
        if p_xcntrc.curr_liab < 1000 or
           p_xcntrc.curr_liab > 9999 or
           p_xcntrc.curr_liab is null
        then
            let p_xcntrc.curr_liab = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "curr_liab"
            return
        end if
      when scr_fld = "long_term_liab"
        if p_xcntrc.long_term_liab < 1000 or
           p_xcntrc.long_term_liab > 9999 or
           p_xcntrc.long_term_liab is null
        then
            let p_xcntrc.long_term_liab = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "long_term_liab"
            return
        end if
      when scr_fld = "capital"
        if p_xcntrc.capital < 1000 or
           p_xcntrc.capital > 9999 or
           p_xcntrc.capital is null
        then
            let p_xcntrc.capital = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "capital"
            return
        end if
      when scr_fld = "income"
        if p_xcntrc.income < 1000 or
           p_xcntrc.income > 9999 or
           p_xcntrc.income is null
        then
            let p_xcntrc.income = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "income"
            return
        end if
      when scr_fld = "cost_goods"
        if p_xcntrc.cost_goods < 1000 or
           p_xcntrc.cost_goods > 9999 or
           p_xcntrc.cost_goods is null
        then
            let p_xcntrc.cost_goods = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "cost_goods"
            return
        end if
      when scr_fld = "expense"
        if p_xcntrc.expense < 1000 or
           p_xcntrc.expense > 9999 or
           p_xcntrc.expense is null
        then
            let p_xcntrc.expense = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "expense"
            return
        end if
    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # This (among other things) sets data_changed
    call lib_after()

end function
# llh_a_field()

######################################################################
function llh_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#
    if mustfill(p_xcntrc.curr_asset,"Current Asset")
      then let nxt_fld = "curr_asset" return end if
    if mustfill(p_xcntrc.fixed_asset,"Fixed Asset")
      then let nxt_fld = "fixed_asset" return end if
    if mustfill(p_xcntrc.curr_liab,"Current Liability")
      then let nxt_fld = "curr_liab" return end if
    if mustfill(p_xcntrc.long_term_liab,"Long Term Liability")
      then let nxt_fld = "long_term_liab" return end if
    if mustfill(p_xcntrc.capital,"Capital")
      then let nxt_fld = "capital" return end if
    if mustfill(p_xcntrc.income,"Income")
      then let nxt_fld = "income" return end if
    if mustfill(p_xcntrc.cost_goods,"Cost of Goods")
      then let nxt_fld = "cost_goods" return end if
    if mustfill(p_xcntrc.expense,"Expense")
      then let nxt_fld = "expense" return end if

    if nxt_fld is null then call chk_range() end if

end function
# llh_a_input()

######################################################################
function llh_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# llh_event()

######################################################################
function llh_setdata()
######################################################################
# This function is called to set the this_data global variable.
#
    case
      when scr_fld = "curr_asset"
        call set_data(p_xcntrc.curr_asset)
      when scr_fld = "fixed_asset"
        call set_data(p_xcntrc.fixed_asset)
      when scr_fld = "curr_liab"
        call set_data(p_xcntrc.curr_liab)
      when scr_fld = "long_term_liab"
        call set_data(p_xcntrc.long_term_liab)
      when scr_fld = "capital"
        call set_data(p_xcntrc.capital)
      when scr_fld = "income"
        call set_data(p_xcntrc.income)
      when scr_fld = "cost_goods"
        call set_data(p_xcntrc.cost_goods)
      when scr_fld = "expense"
        call set_data(p_xcntrc.expense)
    end case
end function
# llh_setdata()

######################################################################
function llh_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "curr_asset"
        display by name p_xcntrc.curr_asset attribute(reverse)
      when scr_fld = "fixed_asset"
        display by name p_xcntrc.fixed_asset attribute(reverse)
      when scr_fld = "curr_liab"
        display by name p_xcntrc.curr_liab attribute(reverse)
      when scr_fld = "long_term_liab"
        display by name p_xcntrc.long_term_liab attribute(reverse)
      when scr_fld = "capital"
        display by name p_xcntrc.capital attribute(reverse)
      when scr_fld = "income"
        display by name p_xcntrc.income attribute(reverse)
      when scr_fld = "cost_goods"
        display by name p_xcntrc.cost_goods attribute(reverse)
      when scr_fld = "expense"
        display by name p_xcntrc.expense attribute(reverse)
    end case
end function
# llh_high()

######################################################################
function llh_display()
######################################################################
# This function displays all p_* variables onto the screen
#
    display p_xcntrc.* to s_cntrc.*  attribute(red)
end function
# llh_display()

######################################################################
function llh_p_prep()
######################################################################
# This function creates the p_* record from the m_* record
#
    let p_xcntrc.curr_asset = m_xcntrc.curr_asset
    let p_xcntrc.fixed_asset = m_xcntrc.fixed_asset
    let p_xcntrc.curr_liab = m_xcntrc.curr_liab
    let p_xcntrc.long_term_liab = m_xcntrc.long_term_liab
    let p_xcntrc.capital = m_xcntrc.capital
    let p_xcntrc.income = m_xcntrc.income
    let p_xcntrc.cost_goods = m_xcntrc.cost_goods
    let p_xcntrc.expense = m_xcntrc.expense

end function
# llh_p_prep()

######################################################################
function llh_m_prep()
######################################################################
# This function creates the m_* record from the p_* record
#
    let m_xcntrc.curr_asset = p_xcntrc.curr_asset
    let m_xcntrc.fixed_asset = p_xcntrc.fixed_asset
    let m_xcntrc.curr_liab = p_xcntrc.curr_liab
    let m_xcntrc.long_term_liab = p_xcntrc.long_term_liab
    let m_xcntrc.capital = p_xcntrc.capital
    let m_xcntrc.income = p_xcntrc.income
    let m_xcntrc.cost_goods = p_xcntrc.cost_goods
    let m_xcntrc.expense = p_xcntrc.expense
end function
# llh_m_prep()

######################################################################
function llh_read(ring_rowid)
######################################################################
# This function loads the header p_* record elements.
# It first loads the m_* record from disk, then it
# calls the p_prep routine to fill the p_* record elements.
#
    define
        ring_rowid  integer     # rowid of record to read

    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch = "select * from stxcntrc where rowid = ?"
        prepare read_specs from scratch
        declare h_cursor cursor for read_specs
    end if

    # Read the header record
    open h_cursor using ring_rowid
    fetch h_cursor into m_xcntrc.*
    if sqlca.sqlcode != 0 then return end if
    let q_xcntrc.row_id = ring_rowid
    close h_cursor
    call llh_p_prep()
end function
# llh_read()

######################################################################
function llh_delete(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    delete from stxcntrc
        where rowid = ring_rowid
end function
# llh_delete()

######################################################################
function llh_add()
######################################################################
# This function inserts data into the header table.
#
    define
        new_rowid integer  # Rowid after insert
    # Insert the data
    insert into stxcntrc values(m_xcntrc.*)
    let new_rowid = sqlca.sqlerrd[6]

    # Reset rowid
    let sqlca.sqlerrd[6] = new_rowid

end function
# llh_add()

######################################################################
function llh_update(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    update stxcntrc set stxcntrc.* = m_xcntrc.*
      where rowid = ring_rowid
end function
# llh_update()


#######################################################################
function chk_range()
#   returns true/false based upon good/bad data in screen
#######################################################################
#   this function is called upon exiting the input statement
#   to make sure the account ranges are in ascending order.
#
    if
      p_xcntrc.curr_asset < p_xcntrc.fixed_asset and
      p_xcntrc.fixed_asset < p_xcntrc.curr_liab and
      p_xcntrc.curr_liab < p_xcntrc.long_term_liab and
      p_xcntrc.long_term_liab < p_xcntrc.capital and
      p_xcntrc.capital < p_xcntrc.income and
      p_xcntrc.income < p_xcntrc.cost_goods and
      p_xcntrc.cost_goods < p_xcntrc.expense
    then
        # accounts are properly ordered
    else
        call fg_err(1) # Account ranges must be ascending
        let nxt_fld = "curr_asset"
    end if
end function
# chk_range()

#######################################################################
function ok_add()
# returning true/false based upon ok/not_ok to add
#######################################################################
#
    call fg_err(32) #This function has been disabled
    return false
end function
# ok_add()

#######################################################################
function ok_find()
# returning true/false based upon ok/not_ok to find
#######################################################################
#
    call fg_err(32) #This function has been disabled
    return false
end function
# ok_find()

#######################################################################
function ok_delete()
# returning true/false based upon ok/not_ok to delete
#######################################################################
#
    call fg_err(32) #This function has been disabled
    return false
end function
# ok_delete()

######################################################################
function cash_warning(new_acct_no)
# returns true/false depending on whether a cash account exists
######################################################################
#  this function is called when the user attempts to change the
#  current asset account range if it will exclude the cash account
#  setup previously in the i_cashe program.  this is necessary due to
#  the fact that no updating is allowed to the i_cashe records.

    define
          new_acct_no like stxckrgr.acct_no,
          p_count integer

    select count(*) into p_count from stxckrgr
       where stxckrgr.acct_no < new_acct_no

    # no checking accounts setup that conflict with range change
    if p_count < 1 then return false end if

    # otherwise return true, at least one checking account exists
    return true

end function
# cash_warning

