######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    dup smallint, # (boolean) duplicate account number entry?
    dup_stmt char(128), # prepared variable used by check_dup function
    dup_prep char(1),  # flag: has check_dup function has been prepped
    lookup_prep char(1), # Have the lookups been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=field
    tab_pressed smallint # (boolean) was the tab key pressed?


######################################################################
function llh_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    # Trap fatal errors
    whenever error call error_handler

    # Initialize some variables
    let exit_level = 0
    let tab_pressed = false

    let dup = false


    input p_xchrtr.* without defaults from s_chrtr.*

      # All entry fields must have before and after field processing
      before field acct_no call llh_b_field("stxchrtr.acct_no")
        goto next_field
      before field acct_desc call llh_b_field("stxchrtr.acct_desc")
        goto next_field
      before field subtotal_group call llh_b_field("stxchrtr.subtotal_group")
        goto next_field
      before field incr_with_crdt call llh_b_field("stxchrtr.incr_with_crdt")
        goto next_field

      after field acct_no call llh_a_field()
        goto next_field
      after field acct_desc call llh_a_field()
        goto next_field
      after field subtotal_group call llh_a_field()
        goto next_field
      after field incr_with_crdt call llh_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 1 then call llh_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after input' logic before exiting input
        call llh_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_key("llh_input")  # Map the key to the event
        call llh_event()

      # Programmatic next field logic
      label next_field:
        let scratch = nxt_fld
        let nxt_fld = null
        case
          when scratch is null  # No need to go through 'case'
          when scratch = "acct_no" next field acct_no
          when scratch = "acct_desc" next field acct_desc
          when scratch = "subtotal_group" next field subtotal_group
          when scratch = "incr_with_crdt" next field incr_with_crdt
          when scratch = "exit input" goto end_input
        end case
    end input

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# llh_input()

######################################################################
function llh_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field level exit
    let exit_level = 1

    # Specify fields only if work needs to be done.
    case
      when scr_fld = "acct_no"
        if not dup and activity()
        then
            let nxt_fld = "acct_desc"
        end if
        if cash_warning(p_xchrtr.acct_no)
        then
            let nxt_fld = "acct_desc"
        end if

      when scr_fld = "incr_with_crdt"
        if activity()
        then
            let nxt_fld = "acct_no"
        end if

    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# llh_b_field()

######################################################################
function llh_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # No more exit levels required
    let exit_level = 0

    # After field logic
    case
      when scr_fld = "acct_no"
        if p_xchrtr.acct_no < 1000 or
           p_xchrtr.acct_no > 9999 or
           p_xchrtr.acct_no is null
        then
            let p_xchrtr.acct_no = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "acct_no"
            return
        end if

        if check_dup()
        then
            let dup = true
            let nxt_fld = "acct_no"
        else
            let dup = false
            call setype()
        end if

      when scr_fld = "incr_with_crdt"
        if (p_xchrtr.incr_with_crdt != "Y" and
           p_xchrtr.incr_with_crdt != "N") or
           p_xchrtr.incr_with_crdt is null
        then
            let p_xchrtr.incr_with_crdt = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "incr_with_crdt"
            return
        else
            call setype()
        end if
    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # This (among other things) sets data_changed
    call lib_after()

end function
# llh_a_field()

######################################################################
function llh_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#
    if mustfill(p_xchrtr.acct_no,"Account Number")
        then let nxt_fld = "acct_no" return end if
    if mustfill(p_xchrtr.acct_desc,"Account Description")
        then let nxt_fld = "acct_desc" return end if
    if mustfill(p_xchrtr.incr_with_crdt,"Increase w/Credit")
        then let nxt_fld = "incr_with_crdt" return end if


end function
# llh_a_input()

######################################################################
function llh_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# llh_event()

######################################################################
function llh_setdata()
######################################################################
# This function is called to set the this_data global variable.
#
    case
      when scr_fld = "acct_no"
        call set_data(p_xchrtr.acct_no)
      when scr_fld = "acct_desc"
        call set_data(p_xchrtr.acct_desc)
      when scr_fld = "subtotal_group"
        call set_data(p_xchrtr.subtotal_group)
      when scr_fld = "incr_with_crdt"
        call set_data(p_xchrtr.incr_with_crdt)
    end case
end function
# llh_setdata()

######################################################################
function llh_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "acct_no"
        display by name p_xchrtr.acct_no attribute(reverse)
      when scr_fld = "acct_desc"
        display by name p_xchrtr.acct_desc attribute(reverse)
      when scr_fld = "subtotal_group"
        display by name p_xchrtr.subtotal_group attribute(reverse)
      when scr_fld = "incr_with_crdt"
        display by name p_xchrtr.incr_with_crdt attribute(reverse)
    end case
end function
# llh_high()

######################################################################
function llh_display()
######################################################################
# This function displays all p_* variables onto the screen
#
    display p_xchrtr.* to s_chrtr.*  attribute(red)
end function
# llh_display()

######################################################################
function llh_p_prep()
######################################################################
# This function creates the p_* record from the m_* record
#
    let p_xchrtr.acct_no = m_xchrtr.acct_no
    let p_xchrtr.acct_type = m_xchrtr.acct_type
    let p_xchrtr.acct_desc = m_xchrtr.acct_desc
    let p_xchrtr.subtotal_group = m_xchrtr.subtotal_group
    let p_xchrtr.incr_with_crdt = m_xchrtr.incr_with_crdt

end function
# llh_p_prep()

######################################################################
function llh_m_prep()
######################################################################
# This function creates the m_* record from the p_* record
#
    let m_xchrtr.acct_no = p_xchrtr.acct_no
    let m_xchrtr.acct_type = p_xchrtr.acct_type
    let m_xchrtr.acct_desc = p_xchrtr.acct_desc
    let m_xchrtr.subtotal_group = p_xchrtr.subtotal_group
    let m_xchrtr.incr_with_crdt = p_xchrtr.incr_with_crdt
end function
# llh_m_prep()

######################################################################
function llh_read(ring_rowid)
######################################################################
# This function loads the header p_* record elements.
# It first loads the m_* record from disk, then it
# calls the p_prep routine to fill the p_* record elements.
#
    define
        ring_rowid  integer     # rowid of record to read

    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch = "select * from stxchrtr where rowid = ?"
        prepare read_specs from scratch
        declare h_cursor cursor for read_specs
    end if

    # Read the header record
    open h_cursor using ring_rowid
    fetch h_cursor into m_xchrtr.*
    if sqlca.sqlcode != 0 then return end if
    let q_xchrtr.row_id = ring_rowid
    close h_cursor
    call llh_p_prep()
end function
# llh_read()

######################################################################
function llh_delete(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    delete from stxchrtr
        where rowid = ring_rowid
end function
# llh_delete()

######################################################################
function llh_add()
######################################################################
# This function inserts data into the header table.
#
    define
        new_rowid integer  # Rowid after insert
    # Insert the data
    insert into stxchrtr values(m_xchrtr.*)
    let new_rowid = sqlca.sqlerrd[6]

    # Reset rowid
    let sqlca.sqlerrd[6] = new_rowid

end function
# llh_add()

######################################################################
function llh_update(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    update stxchrtr set stxchrtr.* = m_xchrtr.*
      where rowid = ring_rowid
end function
# llh_update()


#######################################################################
function ok_delete()
# returning true/false for  ok/not_ok  to delete
#######################################################################
#
    if activity()
    then
        call fg_err(2) # Cannot Delete Account With Activity
        return false
    end if

    if cash_warning(p_xchrtr.acct_no)
    then
        call fg_err(3) # Cannot Delete Checking Account
        return false
    end if

    return true
end function
# ok_delete()

#####################################################################
function check_dup()
# checks for duplicate value in a table (used with input statements)
# returns true if a duplicate value is found, otherwise false
#####################################################################

define
    p_rowid smallint,
    p_count smallint

    if dup_prep is null
    then
        let dup_prep = "Y"
        let scratch = "select count(*) from stxchrtr ",
                      "where acct_no = ? and rowid != ?"
        prepare dup_stmt from scratch
        declare dup_cur cursor for dup_stmt
    end if

    let p_rowid = get_scrlib("curs_rowid")

    open dup_cur using p_xchrtr.acct_no, p_rowid
    fetch dup_cur into p_count
    close dup_cur

    if p_count > 0
    then
      call fg_err(1) # Duplicate Account Number
      return true
    end if

    return false

end function
# check_dup()

#######################################################################
function setype()
#######################################################################
# - this function takes the account number and figures the acct type.
    if p_xchrtr.acct_no < m_xcntrc.fixed_asset then
        call setfld("CURRENT ASSETS","A","1","N") return end if
    if p_xchrtr.acct_no < m_xcntrc.curr_liab then
        call setfld("FIXED ASSETS","A","2","N") return end if
    if p_xchrtr.acct_no < m_xcntrc.long_term_liab then
        call setfld("CUR LIABILITIES","B","3","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.capital then
        call setfld("L/T LIABILITIES","B","4","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.income then
        call setfld("CAPITAL","C","5","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.cost_goods then
        call setfld("INCOME","D","6","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.expense then
        call setfld("COST OF GOODS","E","7","N") return end if
    call setfld("EXPENSES","F","8","N")
end function
# setype()

#######################################################################
function setfld(acct_type,acct_cat,processing_seq,incr_with_crdt)
#######################################################################
#
    define
        acct_type   like stxchrtr.acct_type,
        acct_cat    like stxchrtr.acct_cat,
        processing_seq  like stxchrtr.processing_seq,
        incr_with_crdt  like stxchrtr.incr_with_crdt

    let p_xchrtr.acct_type = acct_type
    display by name p_xchrtr.acct_type
    let m_xchrtr.acct_cat = acct_cat
    let m_xchrtr.processing_seq = processing_seq
    if p_xchrtr.incr_with_crdt is null
    then
        let p_xchrtr.incr_with_crdt = incr_with_crdt
        display by name p_xchrtr.incr_with_crdt
    end if
end function
# setfld()

#######################################################################
function activity()
# returning true/false based on activity/no_activity posted.
#######################################################################
#
    define
        counter smallint

    # if account number is null, return false
    if p_xchrtr.acct_no is null then return false end if

    # check for activity in history file
    select count(*) into counter from stxchrtd
            where acct_no = p_xchrtr.acct_no
    if counter > 0
    then
        return true
    end if
    # check for activity in activity file
    select count(*) into counter from stgactvd
            where acct_no = p_xchrtr.acct_no
    if counter > 0
    then
        return true
    end if
    # can't find any activity
    return false
end function
# activity()

######################################################################
function cash_warning(new_acct_no)
# returns true/false depending on whether a cash account exists
######################################################################
#  this function is called when the user attempts to change the
#  account number if a checking account exsists for it in the i_cashe
#  program.  this is necessary due to the fact that no updating is
#  allowed to the i_cashe records.

    define
          new_acct_no like stxckrgr.acct_no,
          p_count integer

    select count(*) into p_count from stxckrgr
       where stxckrgr.acct_no = new_acct_no

    # no checking accounts exsist using that account number
    if p_count < 1 then return false end if

    # otherwise return true, at least one checking account exists
    return true

end function
# cash_warning

