########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/i_chart.4gs/screen1.trg  1.3  Delta: 2/20/91
########################################################################

defaults
    define
        m_xcntrc record like stxcntrc.*,  # record like the xcontrol file
        ;

    after_init
        call xcontrol() returning m_xcntrc.*;

input 1
    static_define
        dup smallint, # (boolean) duplicate account number entry?
        dup_stmt char(128), # prepared variable used by check_dup function
        dup_prep char(1),   # flag: has check_dup function has been prepped
        ;

    before_input
        let dup = false
        ;

    before_field acct_no
        if not dup and activity()
        then
            let nxt_fld = "acct_desc"
        end if
        if cash_warning(p_xchrtr.acct_no)
        then
            let nxt_fld = "acct_desc"
        end if
        ;

    after_field acct_no
        if p_xchrtr.acct_no < 1000 or
           p_xchrtr.acct_no > 9999 or
           p_xchrtr.acct_no is null 
        then 
            let p_xchrtr.acct_no = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "acct_no"
            return
        end if

        if check_dup()
        then
            let dup = true
            let nxt_fld = "acct_no"
        else
            let dup = false
            call setype()
        end if
        ;

    before_field incr_with_crdt
        if activity()
        then
            let nxt_fld = "acct_no"
        end if
        ;

    after_field incr_with_crdt
        if (p_xchrtr.incr_with_crdt != "Y" and
           p_xchrtr.incr_with_crdt != "N") or
           p_xchrtr.incr_with_crdt is null
        then 
            let p_xchrtr.incr_with_crdt = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "incr_with_crdt"
            return
        else 
            call setype()
        end if;


    after_input
        if mustfill(p_xchrtr.acct_no,"Account Number")
            then let nxt_fld = "acct_no" return end if
        if mustfill(p_xchrtr.acct_desc,"Account Description")
            then let nxt_fld = "acct_desc" return end if
        if mustfill(p_xchrtr.incr_with_crdt,"Increase w/Credit")
            then let nxt_fld = "incr_with_crdt" return end if
        ;

    at_eof
#######################################################################
function ok_delete()
# returning true/false for  ok/not_ok  to delete
#######################################################################
#
    if activity()
    then
        call fg_err(2) # Cannot Delete Account With Activity
        return false
    end if

    if cash_warning(p_xchrtr.acct_no)
    then
        call fg_err(3) # Cannot Delete Checking Account
        return false
    end if

    return true
end function
# ok_delete()

#####################################################################
function check_dup()
# checks for duplicate value in a table (used with input statements)
# returns true if a duplicate value is found, otherwise false
#####################################################################

define
    p_rowid smallint,
    p_count smallint

    if dup_prep is null
    then
        let dup_prep = "Y"
        let scratch = "select count(*) from stxchrtr ",
                      "where acct_no = ? and rowid != ?"
        prepare dup_stmt from scratch
        declare dup_cur cursor for dup_stmt
    end if

    let p_rowid = get_scrlib("curs_rowid")

    open dup_cur using p_xchrtr.acct_no, p_rowid
    fetch dup_cur into p_count
    close dup_cur

    if p_count > 0
    then
      call fg_err(1) # Duplicate Account Number
      return true
    end if

    return false
   
end function
# check_dup()

#######################################################################
function setype()
#######################################################################
# - this function takes the account number and figures the acct type.
    if p_xchrtr.acct_no < m_xcntrc.fixed_asset then
        call setfld("CURRENT ASSETS","A","1","N") return end if
    if p_xchrtr.acct_no < m_xcntrc.curr_liab then
        call setfld("FIXED ASSETS","A","2","N") return end if
    if p_xchrtr.acct_no < m_xcntrc.long_term_liab then
        call setfld("CUR LIABILITIES","B","3","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.capital then
        call setfld("L/T LIABILITIES","B","4","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.income then
        call setfld("CAPITAL","C","5","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.cost_goods then
        call setfld("INCOME","D","6","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.expense then
        call setfld("COST OF GOODS","E","7","N") return end if
    call setfld("EXPENSES","F","8","N")
end function
# setype()

#######################################################################
function setfld(acct_type,acct_cat,processing_seq,incr_with_crdt)
#######################################################################
#
    define
        acct_type   like stxchrtr.acct_type,
        acct_cat    like stxchrtr.acct_cat,
        processing_seq  like stxchrtr.processing_seq,
        incr_with_crdt  like stxchrtr.incr_with_crdt

    let p_xchrtr.acct_type = acct_type
    display by name p_xchrtr.acct_type
    let m_xchrtr.acct_cat = acct_cat
    let m_xchrtr.processing_seq = processing_seq
    if p_xchrtr.incr_with_crdt is null
    then
        let p_xchrtr.incr_with_crdt = incr_with_crdt
        display by name p_xchrtr.incr_with_crdt
    end if
end function
# setfld()

#######################################################################
function activity()
# returning true/false based on activity/no_activity posted.
#######################################################################
#
    define
        counter smallint

    # if account number is null, return false
    if p_xchrtr.acct_no is null then return false end if

    # check for activity in history file
    select count(*) into counter from stxchrtd
            where acct_no = p_xchrtr.acct_no
    if counter > 0
    then
        return true
    end if
    # check for activity in activity file
    select count(*) into counter from stgactvd
            where acct_no = p_xchrtr.acct_no
    if counter > 0
    then
        return true
    end if
    # can't find any activity
    return false
end function
# activity()

######################################################################
function cash_warning(new_acct_no)
# returns true/false depending on whether a cash account exists
######################################################################
#  this function is called when the user attempts to change the
#  account number if a checking account exsists for it in the i_cashe
#  program.  this is necessary due to the fact that no updating is
#  allowed to the i_cashe records.

    define
          new_acct_no like stxckrgr.acct_no,
          p_count integer

    select count(*) into p_count from stxckrgr
       where stxckrgr.acct_no = new_acct_no

    # no checking accounts exsist using that account number
    if p_count < 1 then return false end if

    # otherwise return true, at least one checking account exists
    return true

end function
# cash_warning
        ;
