########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/i_chglog.4gs/main.4gl  1.11  Delta: 2/13/91
########################################################################

globals "globals.4gl"

#######################################################################
main
#######################################################################
# Trap fatal errors
# Fool the linker to link in scr library stuff
#
    whenever error call error_handler
    if false then if event_prc("","") then end if end if
    options input wrap
    options prompt line first + 1
    options form line first + 3
    let progid = "all.i_chglog"
    clear screen
    open window win1 at 2,3 with 22 rows, 76 columns
        attribute (border, red)
    clear window win1
    open window win2 at 2,3 with 22 rows, 76 columns
        attribute (blue)
    open form screen1 from "screen1"
    display form screen1
    call get_logpath()
    call update_logpath()
    exit program (0)

end main

#######################################################################
function get_logpath()
#######################################################################
#
# this function is used to get the logpath assigned to the specified
# database.  if the system is using on-line or turbo, the log tracking
# must be done by the system administrator.
# the function will get the logpath from stxcompr for the specified
# database first, if the system is using informix 4.0 or later, the
# function will get the logfile from the table systables.
#
    define
        n smallint           # working number

    # default to the standard database
    let compid  = null
    let dbname  = null
    let logpath = null

    # parse the command line for "-c $company" where company is
    # the company id that is used as a key into stxcompr to find
    # the database name, and the logfile path.
    for n = 1 to num_args() - 1
        if arg_val(n) = "-c"
        then
            let compid = arg_val(n+1)
        end if
        if arg_val(n) = "-f"
        then
            let fg_variable = arg_val(n+1)
        end if
    end for

    # try to find the database name in stxcompr
    if compid is not null
    then
        let no_company = false
        select db_name, logfile into dbname, logpath
            from stxcompr where comp_id = compid
        if sqlca.sqlcode = notfound
        then
            let no_company = true
            # try opening the database to see if it exists
            whenever error continue
            database compid
            whenever error call error_handler
            if sqlca.sqlcode = 0
            then 
                let dbname = compid
            else 
                exit program(1)
            end if
        end if
    end if

    close database
    database dbname

    # check the table syscolumns to see if the column dirpath exists in
    # systables.  if it does not, then either turbo or on-line 
    # are in use and it is up to the system administrator to 
    # turn logging on and off.  

    if is_turbo()
    then 
        call no_can_do()
        exit program(1)
    end if

    # check systables for the column viewtext in table sysviews.  if it
    # exists, then a 4.0 or later standard engine is in use and logging
    # is always turned on.  if 4.0 or later standard engine is used then
    # use the dirpath in systables as the logpath

    if is_4_0()
    then
        select dirpath into p_logpath from systables 
           where tabname = "syslog"
        if sqlca.sqlcode = 0   # if dirpath is found in systables
        then
            let logpath = p_logpath
        end if 
    end if

end function
# get_logpath

#######################################################################
function update_logpath()
#######################################################################
#
# this function allows the user to update the path of the logfile.
# this function will start # the database with the new log file 
# and update the logfile column in
# the stxcompr table of the "standard" database.
#
    define 
      sql_stmt char(128),
      quote char(1)

    let quote = "\""

    if logpath is null
    then
        let logpath = fg_variable clipped, 
               "/accounting/data/", dbname[1,10] clipped,".log"
    end if
    let p_logpath = logpath

    display "Update:  [ESC] to Store  [DEL] to Cancel" at 1,2
    display "Enter changes into form" at 2,2
    call menu_line()
    display logpath, p_logpath 
        to s_screen.oldlogfile, s_screen.logfile

    input p_logpath without defaults from s_screen.logfile
      after field logfile
        if p_logpath is null
        then
          let p_logpath = fg_variable clipped, 
                 "/accounting/data/", dbname[1,10] clipped,".log"
          display p_logpath to s_screen.logfile
        end if
    end input

    if int_flag != 0 then return end if

    if logpath = p_logpath then return end if

    close database

    # create logfile
    let sql_stmt = 
      "start database ", dbname clipped,
        " with log in ", quote, p_logpath clipped, quote
    prepare start_db from sql_stmt

    whenever error continue
    execute start_db
    # the "start database with log in" command failed
    if sqlca.sqlcode != 0
    then
        call bad_create()
        exit program(1)
    end if
    whenever error call error_handler

    # if not 4.0 then removed the "syslog" row from systables
    if not is_4_0() 
    then
        delete from systables where tabname = "syslog"
    end if

    close database
    database standard
    if no_company
    then
        insert into stxcompr 
          (db_name, logfile, comp_id) values (dbname, p_logpath, compid)
    else
        update stxcompr set logfile = p_logpath where comp_id = compid
    end if

end function
# update_logpath()

######################################################################
function no_can_do()
######################################################################
#
# this function is called if the user is attempting to change the
# logfile of a turbo/online database.
#
    define response char(1)  # key pressed in response

    open window no_win at 7,18 with 12 rows, 46 columns
       attribute(border, red, prompt line 12)

    display "      Turbo Or Online Database Detected.      " at 1,1
       attribute(reverse)
    display "==============================================" at 2,1
    display " The database you are currently using is run- " at 3,1
    display " ning under Turbo or On-Line.  The logfile    " at 4,1
    display " cannot be changed by this program.           " at 5,1
    display " Turbo and On-Line are managed by the         " at 6,1
    display " DB-monitor program, tbmonitor.               " at 7,1
    display "                                              " at 8,1
    display " Contact your system administrator or         " at 9,1
    display " authorized FourGen reseller for assistance.  " at 10,1
    display "==============================================" at 11,1
    prompt  "               Press Any Key...              "
      for char response

    close window no_win

end function
# no_can_do()

######################################################################
function bad_create()
######################################################################
#
# this function is called if the user is attempting to change the
# logfile to a logfile that alread exist.
#
    define response char(1)  # key pressed in response

    open window bad_win at 7,18 with 12 rows, 46 columns
       attribute(border, red, prompt line 12)

    display "      Creation of Logfile failed.             " at 1,1
       attribute(reverse)
    display "==============================================" at 2,1
    display " The system was not able to create the logfile" at 3,1
    display " that you have specified.  This usually means " at 4,1
    display " that a logfile already exists with the same  " at 5,1
    display " logfile name.  Use another name for your     " at 6,1
    display " logfile or remove the existing logfile from  " at 7,1
    display " the system and then re-run this program.     " at 8,1
    display "==============================================" at 9,1
    prompt  "               Press Any Key...              "
      for char response

    close window bad_win

end function
# bad_create()
