######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    lookup_prep char(1), # Have the lookups been prepared?
    insert_prep char(1), # Has the insert statement been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=row, 2=field
    tab_pressed smallint,# (boolean) was the tab key pressed?
    in_insert smallint   # (boolean) true if we're in 'insert row'


######################################################################
function lld_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    define
        goto_top  smallint   # (boolean) Go to the top of the array?

    # Trap fatal errors
    whenever error call error_handler

    # Set some initial values
    let goto_top = true

    # This loop provides the ability to goto the top of the array.
    # Logic contained within the loop is not indented for readability.
    while goto_top

    let goto_top = false
    let in_insert = false
    let exit_level = 0
    let tab_pressed = false
    call set_count(rec1_cnt)
    input array p_xinfor without defaults from s_xinfor.*

      # The following section is for array type inputs
      before row
        if in_insert
        then
            # F1/F2 Scenario - fix parallel array if pre-informix 4.0
            if not is_4_0() and rec1_cnt > 0
              then call lld_a_delete() end if
            let in_insert = false  # Reset in_insert flag
        end if
        let scr_fld = ""           # Reset the current field
        let p_cur = arr_curr()     # Set the current array line
        let s_cur = scr_line()     # Set the current screen line
        let rec1_cnt = arr_count() # Set the number of array elements
        call lld_b_row()
        goto next_field
      after row
        call lld_showline()
        call lld_a_row()
        goto next_field
      before insert
        let in_insert = true
        let rec1_cnt = arr_count() # Set the number of array elements
        call lld_b_insert()
        goto next_field
      after insert
        let in_insert = false
        call lld_a_insert()
        goto next_field
      before delete
        call lld_b_delete()
        goto next_field
      after delete
        call lld_a_delete()
        goto next_field

      # All entry fields must have before and after field processing
      before field src_key call lld_b_field("stxinfor.src_key")
        goto next_field
      before field src_desc call lld_b_field("stxinfor.src_desc")
        goto next_field

      after field src_key call lld_a_field()
        goto next_field
      after field src_desc call lld_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 2 then call lld_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after row' logic if necessary
        if exit_level >= 1 then call lld_a_row() end if
        if nxt_fld is not null then goto next_field end if

        # Don't run 'after input' logic if looping to top
        if goto_top then exit input end if

        # Run the 'after input' logic before exiting input
        call lld_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_key("lld_input")  # Map the key to the event
        call lld_event()

      # Programmatic next field logic
      label next_field:
        let scratch = nxt_fld
        let nxt_fld = null
        case
          when scratch is null  # No need to go through 'case'
          when scratch = "src_key" next field src_key
          when scratch = "src_desc" next field src_desc
          when scratch = "exit input" goto end_input
          when scratch = "goto top"
            call mld_arr_count()
            let goto_top = true
            goto end_input
        end case

    end input

    # This is the end of the 'goto top' loop.
    end while

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# lld_input()

######################################################################
function lld_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field and row level exit
    let exit_level = 2

    # Specify fields only if work needs to be done.
    case
      when scr_fld = "src_key"
        if p_xinfor[p_cur].src_key is not null and
           p_xinfor[p_cur].src_key = "000"
          then let nxt_fld = "src_desc" end if

    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# lld_b_field()

######################################################################
function lld_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # Require row level exit
    let exit_level = 1

    # After field logic
    case
      when scr_fld = "src_key"
        # don't allow entry of blank department code if a
        # description has been entered
        if p_xinfor[p_cur].src_desc is not null
        then
           if mustfill(p_xinfor[p_cur].src_key,"Department Code")
            then let nxt_fld = "src_key" return end if
        end if


      when scr_fld = "src_desc"
        # double-check:
        # don't allow entry of blank department code if a
        # description has been entered
        if p_xinfor[p_cur].src_key is null and
           p_xinfor[p_cur].src_desc is not null
        then
           if mustfill(p_xinfor[p_cur].src_key,"Department Code")
            then let nxt_fld = "src_key" return end if
        end if
        # don't allow entry of blank description if a
        # department code has been entered
        if p_xinfor[p_cur].src_key is not null
        then
          if mustfill(p_xinfor[p_cur].src_desc,"Department Description")
          then let nxt_fld = "src_desc" return end if
        end if


    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # This (among other things) sets data_changed
    call lib_after()

    # After data_changed logic
    if data_changed
    then
        case
          when scr_fld = "src_key"
            if p_xinfor[p_cur].src_key is not null
            then
              if lcl_checkdup()
              then
                  let p_xinfor[p_cur].src_key = null
                  display p_xinfor[p_cur].src_key
                       to s_xinfor[s_cur].src_key
                  let nxt_fld = "src_key"
              end if
            end if

        end case
    end if

end function
# lld_a_field()

######################################################################
function lld_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#

    # collapse the detail array upon leaving the detail section
    call collapse_array()

end function
# lld_a_input()

######################################################################
function lld_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# lld_event()

######################################################################
function lld_b_row()
######################################################################
# This function is called before you enter a new row.
#

end function
# lld_b_row()

######################################################################
function lld_a_row()
######################################################################
# This function is called whenever you leave a row.
#
    # No more exit levels required
    let exit_level = 0

end function
# lld_a_row()

######################################################################
function lld_b_insert()
######################################################################
# This function is called before a new row is added to the array.
#
    define
        n smallint   # Generic counter

    # Expand (shift) the parallel array
    for n = (rec1_cnt - 1) to p_cur step -1
        let q_xinfor[n+1].* = q_xinfor[n].*
    end for

    # Blank out the current array element
    initialize q_xinfor[p_cur].* to null

end function
# lld_b_insert()

######################################################################
function lld_a_insert()
######################################################################
# This function is called after a new row has been added to the array.
#

end function
# lld_a_insert()

######################################################################
function lld_b_delete()
######################################################################
# This function is called when the user presses [F2] (delete a row),
# and before the actual array elements have been shifted.
#
    if p_xinfor[p_cur].src_key = "000"
      then let nxt_fld = "goto top" end if


end function
# lld_b_delete()

######################################################################
function lld_a_delete()
######################################################################
# This function is called when the user presses [F2] (delete a row),
# and after the actual array elements have been shifted.
# (rec1_cnt contains the number of elements before the delete)
#
    define
        n smallint   # Generic counter

    # Compress (shift) the parallel array
    for n = (p_cur + 1) to rec1_cnt
        let q_xinfor[n-1].* = q_xinfor[n].*
    end for

    # Blank out the last (duplicate) data element
    initialize p_xinfor[rec1_cnt].* to null
    initialize q_xinfor[rec1_cnt].* to null

    # Perform array math
    call llh_math()
    call llh_display()

end function
# lld_a_delete()

######################################################################
function lld_setdata()
######################################################################
# This function is called to set the this_data variable
#
    case
      when scr_fld = "src_key"
        call set_data(p_xinfor[p_cur].src_key)
      when scr_fld = "src_desc"
        call set_data(p_xinfor[p_cur].src_desc)
    end case
end function
# lld_setdata()

######################################################################
function lld_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "src_key"
        display p_xinfor[p_cur].src_key
          to s_xinfor[s_cur].src_key attribute(reverse)
      when scr_fld = "src_desc"
        display p_xinfor[p_cur].src_desc
          to s_xinfor[s_cur].src_desc attribute(reverse)
    end case
end function
# lld_high()

######################################################################
function lld_showline()
######################################################################
# This function displays the current p_* variables onto the screen line
#
    display p_xinfor[p_cur].* to s_xinfor[s_cur].*
end function
# lld_showline()

######################################################################
function lld_display()
######################################################################
# This function displays the first screenful of data
#
    for p_cur = 1 to scr1_max
        display p_xinfor[p_cur].* to s_xinfor[p_cur].*  attribute(red)
    end for
end function
# lld_display()

######################################################################
function lld_p_prep(n)
######################################################################
# This function creates the p_[]* record from the m_* record
#
    define
        n smallint      # Array element to process

    let p_xinfor[n].src_key = m_xinfor.src_key
    let p_xinfor[n].src_desc = m_xinfor.src_desc

end function
# lld_p_prep()

######################################################################
function lld_add()
######################################################################
# This function inserts data into the detail table.
#
    # Declare an insert cursor for maximum efficiency
    if insert_prep is null
    then
        let insert_prep = "Y"
        declare d_insert cursor for
          insert into stxinfor values(m_xinfor.*)
    end if

    # Insert all array elements
    open d_insert
    for p_cur = 1 to rec1_cnt
        call lld_m_prep(p_cur)
        put d_insert
    end for
    close d_insert

end function
# lld_add()


#######################################################################
function ok_add()
# returning true/false based upon ok/not_ok to add
#######################################################################
#
    call fg_err(32) #This function has been disabled
    return false
end function
# ok_add()

#######################################################################
function ok_find()
# returning true/false based upon ok/not_ok to find
#######################################################################
#
    call fg_err(32) #This function has been disabled
    return false
end function
# ok_find()

#######################################################################
function ok_delete()
# returning true/false based upon ok/not_ok to delete
#######################################################################
#
    call fg_err(32) #This function has been disabled
    return false
end function
# ok_delete()

#######################################################################
function lcl_checkdup()
# checks for duplicates within an input array
# returns true if a duplicate is found, otherwise false
#######################################################################
#
    define
        n   smallint   # index

    let n = 1

    if p_xinfor[p_cur].src_key is null
      then return false end if

    while n <= arr_count()
        if p_xinfor[n].src_key != p_xinfor[p_cur].src_key or
            n = p_cur or p_xinfor[n].src_key is null
        then
          let n = n + 1
          continue while
        else
          call fg_err(2) # Department Code has already been defined
          return true
        end if
    end while

    return false

end function
# lcl_checkdup()

######################################################################
function lld_m_prep(n)
######################################################################
# This function creates the m_* record from the specified element
# of the p_.* array, and from elements of the header that tie the
# detail lines to the header
#
    define
        n smallint      # Array element to process

    # Items from the header...
    let m_xinfor.src_type = "D"

    # Items from the p_ record array...
    let m_xinfor.src_key = p_xinfor[n].src_key
    let m_xinfor.src_desc = p_xinfor[n].src_desc
end function
# lld_m_prep()

######################################################################
function lld_read()
######################################################################
# This function reads in the array elements from disk.
# It first reads the data into the m_* records, then calls
# p_prep for each array element.
#
    define
        src_str like stxinfor.src_type,
        row_id integer  # Rowid of the fetched element

    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch = "select rowid, * from stxinfor ",
          "where stxinfor.src_type = ?"
        prepare read_specs from scratch
        declare d_cursor cursor for read_specs
    end if

    # Read in the detail lines
    let p_cur = 0
    let src_str = "D"
    open d_cursor using src_str
    foreach d_cursor into row_id, m_xinfor.*
        let p_cur = p_cur + 1
        if p_cur > rec1_max
        then
            let sqlca.sqlcode = -1
            return
        end if
        call lld_p_prep(p_cur)
        let q_xinfor[p_cur].row_id = row_id
    end foreach
    close d_cursor
    let rec1_cnt = p_cur
    call set_count(rec1_cnt)
end function
# lld_read()

######################################################################
function lld_delete()
######################################################################
# This function deletes the rows in the detail table
# that match the where clause
#
    delete from stxinfor
      where stxinfor.src_type = "D"
end function
# lld_delete()

######################################################################
function collapse_array()
######################################################################
#  used to validate all detail rows before they are stored.

    define
          m       integer,   # Working number
          n       integer,   # Working number
          num_out integer    # Working number

    # initialize variables
    let m = 0
    let num_out = 0

    for n = 1 to rec1_cnt
      # criteria for validation
      if p_xinfor[n].src_key is null or
         p_xinfor[n].src_desc is null
      then
          let num_out = num_out + 1
          continue for
      end if
      let m = m + 1
      let p_xinfor[m].* = p_xinfor[m + num_out].*
      let q_xinfor[m].* = q_xinfor[m + num_out].*
    end for
    initialize p_xinfor[m + 1].* to null
    initialize q_xinfor[m + 1].* to null
    for n = (m + 2) to rec1_cnt
      let p_xinfor[n].* = p_xinfor[m + 1].*
      let q_xinfor[n].* = q_xinfor[m + 1].*
    end for
    let rec1_cnt = m

end function
# collapse_array

