######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    lookup_prep char(1), # Have the lookups been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=field
    tab_pressed smallint # (boolean) was the tab key pressed?


######################################################################
function llh_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    # Trap fatal errors
    whenever error call error_handler

    # Initialize some variables
    let exit_level = 0
    let tab_pressed = false


    input p_xcntrc.* without defaults from s_xcntrc.*

      # All entry fields must have before and after field processing
      before field co_name call llh_b_field("stxcntrc.co_name")
        goto next_field
      before field addr1 call llh_b_field("stxcntrc.addr1")
        goto next_field
      before field addr2 call llh_b_field("stxcntrc.addr2")
        goto next_field
      before field city call llh_b_field("stxcntrc.city")
        goto next_field
      before field state call llh_b_field("stxcntrc.state")
        goto next_field
      before field zip call llh_b_field("stxcntrc.zip")
        goto next_field
      before field county call llh_b_field("stxcntrc.county")
        goto next_field
      before field country call llh_b_field("stxcntrc.country")
        goto next_field

      after field co_name call llh_a_field()
        goto next_field
      after field addr1 call llh_a_field()
        goto next_field
      after field addr2 call llh_a_field()
        goto next_field
      after field city call llh_a_field()
        goto next_field
      after field state call llh_a_field()
        goto next_field
      after field zip call llh_a_field()
        goto next_field
      after field county call llh_a_field()
        goto next_field
      after field country call llh_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 1 then call llh_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after input' logic before exiting input
        call llh_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_key("llh_input")  # Map the key to the event
        call llh_event()

      # Programmatic next field logic
      label next_field:
        let scratch = nxt_fld
        let nxt_fld = null
        case
          when scratch is null  # No need to go through 'case'
          when scratch = "co_name" next field co_name
          when scratch = "addr1" next field addr1
          when scratch = "addr2" next field addr2
          when scratch = "city" next field city
          when scratch = "state" next field state
          when scratch = "zip" next field zip
          when scratch = "county" next field county
          when scratch = "country" next field country
          when scratch = "exit input" goto end_input
        end case
    end input

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# llh_input()

######################################################################
function llh_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field level exit
    let exit_level = 1

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# llh_b_field()

######################################################################
function llh_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # No more exit levels required
    let exit_level = 0

    # This (among other things) sets data_changed
    call lib_after()

end function
# llh_a_field()

######################################################################
function llh_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#
    if mustfill(p_xcntrc.co_name, "Company Name")
      then let nxt_fld = "co_name" return end if

end function
# llh_a_input()

######################################################################
function llh_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# llh_event()

######################################################################
function llh_setdata()
######################################################################
# This function is called to set the this_data global variable.
#
    case
      when scr_fld = "co_name"
        call set_data(p_xcntrc.co_name)
      when scr_fld = "addr1"
        call set_data(p_xcntrc.addr1)
      when scr_fld = "addr2"
        call set_data(p_xcntrc.addr2)
      when scr_fld = "city"
        call set_data(p_xcntrc.city)
      when scr_fld = "state"
        call set_data(p_xcntrc.state)
      when scr_fld = "zip"
        call set_data(p_xcntrc.zip)
      when scr_fld = "county"
        call set_data(p_xcntrc.county)
      when scr_fld = "country"
        call set_data(p_xcntrc.country)
    end case
end function
# llh_setdata()

######################################################################
function llh_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "co_name"
        display by name p_xcntrc.co_name attribute(reverse)
      when scr_fld = "addr1"
        display by name p_xcntrc.addr1 attribute(reverse)
      when scr_fld = "addr2"
        display by name p_xcntrc.addr2 attribute(reverse)
      when scr_fld = "city"
        display by name p_xcntrc.city attribute(reverse)
      when scr_fld = "state"
        display by name p_xcntrc.state attribute(reverse)
      when scr_fld = "zip"
        display by name p_xcntrc.zip attribute(reverse)
      when scr_fld = "county"
        display by name p_xcntrc.county attribute(reverse)
      when scr_fld = "country"
        display by name p_xcntrc.country attribute(reverse)
    end case
end function
# llh_high()

######################################################################
function llh_display()
######################################################################
# This function displays all p_* variables onto the screen
#
    display p_xcntrc.* to s_xcntrc.*  attribute(red)
end function
# llh_display()

######################################################################
function llh_p_prep()
######################################################################
# This function creates the p_* record from the m_* record
#
    let p_xcntrc.co_name = m_xcntrc.co_name
    let p_xcntrc.addr1 = m_xcntrc.addr1
    let p_xcntrc.addr2 = m_xcntrc.addr2
    let p_xcntrc.city = m_xcntrc.city
    let p_xcntrc.state = m_xcntrc.state
    let p_xcntrc.zip = m_xcntrc.zip
    let p_xcntrc.county = m_xcntrc.county
    let p_xcntrc.country = m_xcntrc.country

end function
# llh_p_prep()

######################################################################
function llh_m_prep()
######################################################################
# This function creates the m_* record from the p_* record
#
    let m_xcntrc.co_name = p_xcntrc.co_name
    let m_xcntrc.addr1 = p_xcntrc.addr1
    let m_xcntrc.addr2 = p_xcntrc.addr2
    let m_xcntrc.city = p_xcntrc.city
    let m_xcntrc.state = p_xcntrc.state
    let m_xcntrc.zip = p_xcntrc.zip
    let m_xcntrc.county = p_xcntrc.county
    let m_xcntrc.country = p_xcntrc.country
end function
# llh_m_prep()

######################################################################
function llh_read(ring_rowid)
######################################################################
# This function loads the header p_* record elements.
# It first loads the m_* record from disk, then it
# calls the p_prep routine to fill the p_* record elements.
#
    define
        ring_rowid  integer     # rowid of record to read

    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch = "select * from stxcntrc where rowid = ?"
        prepare read_specs from scratch
        declare h_cursor cursor for read_specs
    end if

    # Read the header record
    open h_cursor using ring_rowid
    fetch h_cursor into m_xcntrc.*
    if sqlca.sqlcode != 0 then return end if
    let q_xcntrc.row_id = ring_rowid
    close h_cursor
    call llh_p_prep()
end function
# llh_read()

######################################################################
function llh_delete(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    delete from stxcntrc
        where rowid = ring_rowid
end function
# llh_delete()

######################################################################
function llh_add()
######################################################################
# This function inserts data into the header table.
#
    define
        new_rowid integer  # Rowid after insert
    # Insert the data
    insert into stxcntrc values(m_xcntrc.*)
    let new_rowid = sqlca.sqlerrd[6]

    # Reset rowid
    let sqlca.sqlerrd[6] = new_rowid

end function
# llh_add()

######################################################################
function llh_update(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    update stxcntrc set stxcntrc.* = m_xcntrc.*
      where rowid = ring_rowid
end function
# llh_update()

