########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/i_contrl.4gs/screen1.trg  1.6  Delta: 2/20/91
########################################################################

defaults
    do_not_generate
        lld_read
        lld_m_prep
        lld_delete
        ;

input 1
    after_input
        if mustfill(p_xcntrc.co_name, "Company Name") 
          then let nxt_fld = "co_name" return end if;

input 2
    after_input
        # collapse the detail array upon leaving the detail section
        call collapse_array()
        ;

    before_delete
        if p_xinfor[p_cur].src_key = "000"
          then let nxt_fld = "goto top" end if
        ;

    before_field src_key
        if p_xinfor[p_cur].src_key is not null and
           p_xinfor[p_cur].src_key = "000"
          then let nxt_fld = "src_desc" end if
        ;

    after_field src_key
        # don't allow entry of blank department code if a
        # description has been entered
        if p_xinfor[p_cur].src_desc is not null
        then
           if mustfill(p_xinfor[p_cur].src_key,"Department Code")
            then let nxt_fld = "src_key" return end if
        end if
        ;

    after_field src_desc
        # double-check:
        # don't allow entry of blank department code if a
        # description has been entered
        if p_xinfor[p_cur].src_key is null and
           p_xinfor[p_cur].src_desc is not null
        then
           if mustfill(p_xinfor[p_cur].src_key,"Department Code")
            then let nxt_fld = "src_key" return end if
        end if
        # don't allow entry of blank description if a
        # department code has been entered
        if p_xinfor[p_cur].src_key is not null
        then
          if mustfill(p_xinfor[p_cur].src_desc,"Department Description")
          then let nxt_fld = "src_desc" return end if
        end if
        ;

    after_change_in src_key
        if p_xinfor[p_cur].src_key is not null
        then
          if lcl_checkdup()
          then
              let p_xinfor[p_cur].src_key = null
              display p_xinfor[p_cur].src_key
                   to s_xinfor[s_cur].src_key
              let nxt_fld = "src_key"
          end if
        end if
        ;

    at_eof
#######################################################################
function ok_add()
# returning true/false based upon ok/not_ok to add
#######################################################################
#
    call fg_err(32) #This function has been disabled 
    return false
end function
# ok_add()

#######################################################################
function ok_find()
# returning true/false based upon ok/not_ok to find
#######################################################################
#
    call fg_err(32) #This function has been disabled 
    return false
end function
# ok_find()

#######################################################################
function ok_delete()
# returning true/false based upon ok/not_ok to delete
#######################################################################
#
    call fg_err(32) #This function has been disabled 
    return false
end function
# ok_delete()

#######################################################################
function lcl_checkdup()
# checks for duplicates within an input array
# returns true if a duplicate is found, otherwise false
#######################################################################
#
    define
        n   smallint   # index

    let n = 1

    if p_xinfor[p_cur].src_key is null
      then return false end if

    while n <= arr_count() 
        if p_xinfor[n].src_key != p_xinfor[p_cur].src_key or
            n = p_cur or p_xinfor[n].src_key is null
        then
          let n = n + 1
          continue while
        else
          call fg_err(2) # Department Code has already been defined
          return true
        end if
    end while

    return false

end function
# lcl_checkdup()

######################################################################
function lld_m_prep(n)
######################################################################
# This function creates the m_* record from the specified element
# of the p_.* array, and from elements of the header that tie the
# detail lines to the header
#
    define
        n smallint      # Array element to process

    # Items from the header...
    let m_xinfor.src_type = "D"

    # Items from the p_ record array...
    let m_xinfor.src_key = p_xinfor[n].src_key
    let m_xinfor.src_desc = p_xinfor[n].src_desc
end function
# lld_m_prep()

######################################################################
function lld_read()
######################################################################
# This function reads in the array elements from disk.
# It first reads the data into the m_* records, then calls
# p_prep for each array element.
#
    define
        src_str like stxinfor.src_type,
        row_id integer  # Rowid of the fetched element

    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch = "select rowid, * from stxinfor ",
          "where stxinfor.src_type = ?"
        prepare read_specs from scratch
        declare d_cursor cursor for read_specs
    end if

    # Read in the detail lines
    let p_cur = 0
    let src_str = "D"
    open d_cursor using src_str
    foreach d_cursor into row_id, m_xinfor.*
        let p_cur = p_cur + 1
        if p_cur > rec1_max
        then
            let sqlca.sqlcode = -1
            return
        end if
        call lld_p_prep(p_cur)
        let q_xinfor[p_cur].row_id = row_id
    end foreach
    close d_cursor
    let rec1_cnt = p_cur
    call set_count(rec1_cnt)
end function
# lld_read()

######################################################################
function lld_delete()
######################################################################
# This function deletes the rows in the detail table
# that match the where clause
#
    delete from stxinfor
      where stxinfor.src_type = "D"
end function
# lld_delete()

######################################################################
function collapse_array()
######################################################################
#  used to validate all detail rows before they are stored.

    define
          m       integer,   # Working number
          n       integer,   # Working number
          num_out integer    # Working number

    # initialize variables
    let m = 0
    let num_out = 0

    for n = 1 to rec1_cnt
      # criteria for validation
      if p_xinfor[n].src_key is null or
         p_xinfor[n].src_desc is null
      then
          let num_out = num_out + 1
          continue for
      end if
      let m = m + 1
      let p_xinfor[m].* = p_xinfor[m + num_out].*
      let q_xinfor[m].* = q_xinfor[m + num_out].*
    end for
    initialize p_xinfor[m + 1].* to null
    initialize q_xinfor[m + 1].* to null
    for n = (m + 2) to rec1_cnt
      let p_xinfor[n].* = p_xinfor[m + 1].*
      let q_xinfor[n].* = q_xinfor[m + 1].*
    end for
    let rec1_cnt = m

end function
# collapse_array
        ;
