########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/i_xinfor.4gs/allc.trg  1.3  Delta: 2/20/91
########################################################################

defaults
    do_not_generate
        mlh_construct;

    switchbox_items
        commz commz  # commission code zoom
        deptz deptz  # department code zoom;

    define
        p_sel_filter char(30),            # source type select filter
        p_src_type char(1),               # source type indicator
        m_xcntrc record like stxcntrc.*,  # record like control table;

    after_init
        call get_src_type()
        call t_main();

    at_eof
#####################################################################
function get_src_type()
# determines which screen to use
#####################################################################
# parse the command line for "-t X" where X is the source type.
# if the type is invalid or not found, screen will default to 2
# (all) and the browse filter will default to "1=1".  otherwise,
# the appropriate screen and filter will be chosen.
#
    define
        n smallint,
        src_type char(60)

    for n = 1 to num_args() - 1
        if arg_val(n) = "-t"
        then
            let src_type = arg_val(n+1)
            let p_src_type = upshift(src_type[1])
        end if
    end for
end function
# get_src_type;

input 1
    static_define
        dup_stmt char(128), # prepared variable used by check_dup2 function
        dup_prep char(1),   # flag: has check_dup2 function has been prepped
        ;

    after_change_in src_num_desc
        case
            when p_src_type = "R" or p_src_type = "N"
              if p_xinfor.src_num_desc < 1 or
                 p_xinfor.src_num_desc > 4
              then
                 let p_xinfor.src_num_desc = null
                 call lib_error("lib_scr", "lookup", 1,"")
                 let nxt_fld = "src_num_desc"
              end if
            when p_src_type = "W"
              if p_xinfor.src_num_desc < 0 or
                 p_xinfor.src_num_desc > 99
              then
                 let p_xinfor.src_num_desc = null
                 call lib_error("lib_scr", "lookup", 1, "")
                 let nxt_fld = "src_num_desc"
              end if
        end case
        ;

    before_field src_char_desc
        if p_src_type = "W" or p_src_type = "L"
        then
          let scratch = "zoom"
        end if
        ;

    on_screen_record_prep
        let sqlca.sqlcode = 0
        if p_src_type = "W"
        then
          if alt_lookup("dept_look",false) then end if
        end if
        if p_src_type = "L"
        then
          if alt_lookup("comm_look",false) then end if
        end if
        let sqlca.sqlcode = 0
        ;

    on_disk_record_prep
        if m_xinfor.src_type is null
        then
          let m_xinfor.src_type = p_src_type
        end if
        ;

    after_change_in src_key
        if check_dup2()
        then
            call fg_err(6) # Duplicate reference information row
            let nxt_fld = "src_key"
        end if
        ;

    after_change_in src_char_desc
        case
          when p_src_type = "W"
            # Perform alternate lookup
            if alt_lookup("dept_look",true) = false
              then let nxt_fld = "src_char_desc" end if
          when p_src_type = "L"
            # Perform alternate lookup
            if alt_lookup("comm_look",true) = false
              then let nxt_fld = "src_char_desc" end if
          when p_src_type = "I"
            if (p_xinfor.src_char_desc != "DISCNT" and
               p_xinfor.src_char_desc != "MARKUP" and
               p_xinfor.src_char_desc != "MARKDN") and
               p_xinfor.src_char_desc is not null
            then
                let p_xinfor.src_char_desc = null
                call lib_error("lib_scr", "lookup", 1, "")
                let nxt_fld = "src_char_desc"
            end if
          when p_src_type = "M"
            if (p_xinfor.src_char_desc != "C" and
               p_xinfor.src_char_desc != "V" and
               p_xinfor.src_char_desc != "A") and
               p_xinfor.src_char_desc is not null
            then
               let p_xinfor.src_char_desc = null
               call lib_error("lib_scr", "lookup", 1, "")
               let nxt_fld = "src_char_desc"
            end if
        end case
        ;

    after_change_in src_acct_no
        if p_src_type = "T"
        then
          if p_xinfor.src_acct_no <  m_xcntrc.curr_liab or 
             p_xinfor.src_acct_no >= m_xcntrc.long_term_liab
          then
              call fg_err(2) # Tax Account Not In Current Liability Range
              let p_xinfor.src_acct_no = null
              let p_xinfor.acct_desc = null
              display by name p_xinfor.src_acct_no, p_xinfor.acct_desc
              let nxt_fld = "src_acct_no"
          end if
        end if
        ;

    after_input
        define
             key_var  char(30),  # variable source key title
             desc_var char(30),  # variable source description title
             num_var  char(30),  # variable source number title
             char_var char(30)   # variable source char title
    
        # initialize variable field titles to null
        let key_var = null
        let desc_var = null
        let num_var = null
        let char_var = null
    
        case
            when p_src_type = "B"
              let key_var = "Buyer Code"
              let desc_var = "Buyer Description"
            when p_src_type = "C"
              let key_var = "Commission Code"
              let desc_var = "Commission Description"
              let num_var = "Commission Rate"
            when p_src_type = "D"
              let key_var = "Department Code"
              let desc_var = "Department Description"
            when p_src_type = "I"
              let key_var = "Discount Code"
              let desc_var = "Discount Description"
              let num_var = "Discount Rate"
              let char_var = "Discount Type"
            when p_src_type = "L"
              let key_var = "Salesperson Code"
              let desc_var = "Salesperson Description"
              let char_var = "Commission Code"
            when p_src_type = "M"
              let key_var = "Payment Code"
              let desc_var = "Payment Description"
              let char_var = "Payment Type"
            when p_src_type = "N"
              let key_var = "Reason Code"
              let desc_var = "Reason Description"
              let num_var = "Reason Type"
            when p_src_type = "P"
              let key_var = "Product Class Code"
              let desc_var = "Product Class Description"
            when p_src_type = "R"
              let key_var = "Reason Code"
              let desc_var = "Reason Description"
              let num_var = "Reason Type"
            when p_src_type = "S"
              let key_var = "Source Code"
              let desc_var = "Source Description"
            when p_src_type = "T"
              let key_var = "Tax Code"
              let desc_var = "Tax Description"
              let num_var = "Tax Rate"
            when p_src_type = "W"
              let key_var = "Warehouse Code"
              let desc_var = "Warehouse Description"
              let num_var = "Shipping Lead Time"
              let char_var = "Department"
            otherwise
        end case
    
        if key_var is not null
        then
            if mustfill(p_xinfor.src_key, key_var)
              then let nxt_fld = "src_key" return end if
        end if
    
        if desc_var is not null
        then
            if mustfill(p_xinfor.src_desc, desc_var)
              then let nxt_fld = "src_desc" return end if
        end if
    
        if num_var is not null
        then
            if mustfill(p_xinfor.src_num_desc, num_var)
              then let nxt_fld = "src_num_desc" return end if
        end if
    
        if char_var is not null
        then
            if mustfill(p_xinfor.src_char_desc, char_var)
              then let nxt_fld = "src_char_desc" return end if
        end if
    
        if p_src_type = "T"
        then
            if mustfill(p_xinfor.src_acct_no,"Tax Account Number")
              then let nxt_fld = "src_acct_no" return end if
        end if
        ;

    on_event zoom
        and infield(src_char_desc)
          if p_src_type = "W"
          then
            let scratch =
              " stxinfor.src_type = \"D\""
            if zoom("deptz","(see scratch)")
            then
                let p_xinfor.src_char_desc = scratch
                let nxt_fld = "src_char_desc"
            end if
          end if
          if p_src_type = "L"
          then
            let scratch =
              " stxinfor.src_type = \"C\""
            if zoom("commz","(see scratch)")
            then
                let p_xinfor.src_char_desc = scratch
                let nxt_fld = "src_char_desc"
            end if
          end if
      ;

    at_eof
######################################################################
function mlh_construct()
# returning criteria
######################################################################
#
    construct scratch
      on
        stxinfor.src_key, stxinfor.src_desc
      from
        allc.src_key, allc.src_desc

    let scratch = scratch clipped, " and ", p_sel_filter clipped
    return scratch[1,512]
end function
# mlh_construct

#####################################################################
function check_dup2()
# checks for duplicate key where key consists of two fields
# returns true if a duplicate is found; otherwise returns false
#####################################################################
define
    p_rowid smallint,
    p_count smallint

    if dup_prep is null
    then
        let dup_prep = "Y"
        let scratch = "select count(*) from stxinfor ",
                      "where src_type = ? and ",
                      "src_key = ? and rowid != ?"
        prepare dup_stmt from scratch
        declare dup_cur cursor for dup_stmt
    end if

    let p_rowid = get_scrlib("curs_rowid")
    open dup_cur using p_src_type, p_xinfor.src_key, p_rowid
    fetch dup_cur into p_count
    close dup_cur
    if p_count > 0 then return true end if
    return false
end function
# check_dup2();
