######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    dup_stmt char(128), # prepared variable used by check_dup2 function
    dup_prep char(1),  # flag: has check_dup2 function has been prepped
    lookup_prep char(1), # Have the lookups been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=field
    tab_pressed smallint # (boolean) was the tab key pressed?


######################################################################
function llh_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    # Trap fatal errors
    whenever error call error_handler

    # Initialize some variables
    let exit_level = 0
    let tab_pressed = false


    input p_xinfor.* without defaults from allc.*

      # All entry fields must have before and after field processing
      before field src_key call llh_b_field("stxinfor.src_key")
        goto next_field
      before field src_desc call llh_b_field("stxinfor.src_desc")
        goto next_field
      before field src_num_desc call llh_b_field("stxinfor.src_num_desc")
        goto next_field
      before field src_char_desc call llh_b_field("stxinfor.src_char_desc")
        goto next_field
      before field src_acct_no call llh_b_field("stxinfor.src_acct_no")
        goto next_field

      after field src_key call llh_a_field()
        goto next_field
      after field src_desc call llh_a_field()
        goto next_field
      after field src_num_desc call llh_a_field()
        goto next_field
      after field src_char_desc call llh_a_field()
        goto next_field
      after field src_acct_no call llh_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 1 then call llh_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after input' logic before exiting input
        call llh_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_local("zoom")
        call hot_key("llh_input")  # Map the key to the event
        call llh_event()

      # Programmatic next field logic
      label next_field:
        case
          when nxt_fld is null  # No need to go through 'case'
          when nxt_fld = "src_key" next field src_key
          when nxt_fld = "src_desc" next field src_desc
          when nxt_fld = "src_num_desc" next field src_num_desc
          when nxt_fld = "src_char_desc" next field src_char_desc
          when nxt_fld = "src_acct_no" next field src_acct_no
          when nxt_fld = "exit input" goto end_input
        end case
    end input

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# llh_input()

######################################################################
function llh_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field level exit
    let exit_level = 1

    # Specify fields only if work needs to be done.
    case
      when scr_fld = "src_char_desc"
        if p_src_type = "W" or p_src_type = "L"
        then
          let scratch = "zoom"
        end if

      when scr_fld = "src_acct_no"
    let scratch = "zoom"
    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# llh_b_field()

######################################################################
function llh_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # No more exit levels required
    let exit_level = 0

    # After field logic
    case
      when scr_fld = "src_acct_no"
        let scratch =
          " stxchrtr.acct_cat = \"B\"",
          "and processing_seq = \"3\""
        if auto_zoom("chrtz","stxchrtr","(see scratch)")
          then let p_xinfor.src_acct_no = scratch end if
    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # This (among other things) sets data_changed
    call lib_after()

    # After data_changed logic
    if data_changed
    then
        case
          when scr_fld = "src_key"
            if check_dup2()
            then
                call fg_err(6) # Duplicate reference information row
                let nxt_fld = "src_key"
            end if

          when scr_fld = "src_num_desc"
            case
                when p_src_type = "R" or p_src_type = "N"
                  if p_xinfor.src_num_desc < 1 or
                     p_xinfor.src_num_desc > 4
                  then
                     let p_xinfor.src_num_desc = null
                     call lib_error("lib_scr", "lookup", 1,"")
                     let nxt_fld = "src_num_desc"
                  end if
                when p_src_type = "W"
                  if p_xinfor.src_num_desc < 0 or
                     p_xinfor.src_num_desc > 99
                  then
                     let p_xinfor.src_num_desc = null
                     call lib_error("lib_scr", "lookup", 1, "")
                     let nxt_fld = "src_num_desc"
                  end if
            end case

          when scr_fld = "src_char_desc"
            case
              when p_src_type = "W"
                # Perform alternate lookup
                if alt_lookup("dept_look",true) = false
                  then let nxt_fld = "src_char_desc" end if
              when p_src_type = "L"
                # Perform alternate lookup
                if alt_lookup("comm_look",true) = false
                  then let nxt_fld = "src_char_desc" end if
              when p_src_type = "I"
                if (p_xinfor.src_char_desc != "DISCNT" and
                   p_xinfor.src_char_desc != "MARKUP" and
                   p_xinfor.src_char_desc != "MARKDN") and
                   p_xinfor.src_char_desc is not null
                then
                    let p_xinfor.src_char_desc = null
                    call lib_error("lib_scr", "lookup", 1, "")
                    let nxt_fld = "src_char_desc"
                end if
              when p_src_type = "M"
                if (p_xinfor.src_char_desc != "C" and
                   p_xinfor.src_char_desc != "V" and
                   p_xinfor.src_char_desc != "A") and
                   p_xinfor.src_char_desc is not null
                then
                   let p_xinfor.src_char_desc = null
                   call lib_error("lib_scr", "lookup", 1, "")
                   let nxt_fld = "src_char_desc"
                end if
            end case

          when scr_fld = "src_acct_no"
            # Perform lookups
            if llh_lookup("acct_no_look",true) = false and
              length(this_data) != 0
            then
                let nxt_fld = "src_acct_no"
                return
            end if
            if p_src_type = "T"
            then
              if p_xinfor.src_acct_no <  m_xcntrc.curr_liab or
                 p_xinfor.src_acct_no >= m_xcntrc.long_term_liab
              then
                  call fg_err(2) # Tax Account Not In Current Liability Range
                  let p_xinfor.src_acct_no = null
                  let p_xinfor.acct_desc = null
                  display by name p_xinfor.src_acct_no, p_xinfor.acct_desc
                  let nxt_fld = "src_acct_no"
              end if
            end if

        end case
    end if

end function
# llh_a_field()

######################################################################
function llh_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#
    define
         key_var  char(30),  # variable source key title
         desc_var char(30),  # variable source description title
         num_var  char(30),  # variable source number title
         char_var char(30)   # variable source char title

    # initialize variable field titles to null
    let key_var = null
    let desc_var = null
    let num_var = null
    let char_var = null

    case
        when p_src_type = "B"
          let key_var = "Buyer Code"
          let desc_var = "Buyer Description"
        when p_src_type = "C"
          let key_var = "Commission Code"
          let desc_var = "Commission Description"
          let num_var = "Commission Rate"
        when p_src_type = "D"
          let key_var = "Department Code"
          let desc_var = "Department Description"
        when p_src_type = "I"
          let key_var = "Discount Code"
          let desc_var = "Discount Description"
          let num_var = "Discount Rate"
          let char_var = "Discount Type"
        when p_src_type = "L"
          let key_var = "Salesperson Code"
          let desc_var = "Salesperson Description"
          let char_var = "Commission Code"
        when p_src_type = "M"
          let key_var = "Payment Code"
          let desc_var = "Payment Description"
          let char_var = "Payment Type"
        when p_src_type = "N"
          let key_var = "Reason Code"
          let desc_var = "Reason Description"
          let num_var = "Reason Type"
        when p_src_type = "P"
          let key_var = "Product Class Code"
          let desc_var = "Product Class Description"
        when p_src_type = "R"
          let key_var = "Reason Code"
          let desc_var = "Reason Description"
          let num_var = "Reason Type"
        when p_src_type = "S"
          let key_var = "Source Code"
          let desc_var = "Source Description"
        when p_src_type = "T"
          let key_var = "Tax Code"
          let desc_var = "Tax Description"
          let num_var = "Tax Rate"
        when p_src_type = "W"
          let key_var = "Warehouse Code"
          let desc_var = "Warehouse Description"
          let num_var = "Shipping Lead Time"
          let char_var = "Department"
        otherwise
    end case

    if key_var is not null
    then
        if mustfill(p_xinfor.src_key, key_var)
          then let nxt_fld = "src_key" return end if
    end if

    if desc_var is not null
    then
        if mustfill(p_xinfor.src_desc, desc_var)
          then let nxt_fld = "src_desc" return end if
    end if

    if num_var is not null
    then
        if mustfill(p_xinfor.src_num_desc, num_var)
          then let nxt_fld = "src_num_desc" return end if
    end if

    if char_var is not null
    then
        if mustfill(p_xinfor.src_char_desc, char_var)
          then let nxt_fld = "src_char_desc" return end if
    end if

    if p_src_type = "T"
    then
        if mustfill(p_xinfor.src_acct_no,"Tax Account Number")
          then let nxt_fld = "src_acct_no" return end if
    end if


end function
# llh_a_input()

######################################################################
function llh_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "zoom" and infield(src_acct_no)
        let scratch =
          " stxchrtr.acct_cat = \"B\"",
          "and processing_seq = \"3\""
        if zoom("chrtz","(see scratch)")
        then
            let p_xinfor.src_acct_no = scratch
            let nxt_fld = "src_acct_no"
        end if
      when scr_funct = "zoom"
        and infield(src_char_desc)
          if p_src_type = "W"
          then
            let scratch =
              " stxinfor.src_type = \"D\""
            if zoom("deptz","(see scratch)")
            then
                let p_xinfor.src_char_desc = scratch
                let nxt_fld = "src_char_desc"
            end if
          end if
          if p_src_type = "L"
          then
            let scratch =
              " stxinfor.src_type = \"C\""
            if zoom("commz","(see scratch)")
            then
                let p_xinfor.src_char_desc = scratch
                let nxt_fld = "src_char_desc"
            end if
          end if

      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# llh_event()

######################################################################
function llh_setdata()
######################################################################
# This function is called to set the this_data global variable.
#
    case
      when scr_fld = "src_key"
        call set_data(p_xinfor.src_key)
      when scr_fld = "src_desc"
        call set_data(p_xinfor.src_desc)
      when scr_fld = "src_num_desc"
        call set_data(p_xinfor.src_num_desc)
      when scr_fld = "src_char_desc"
        call set_data(p_xinfor.src_char_desc)
      when scr_fld = "src_acct_no"
        call set_data(p_xinfor.src_acct_no)
    end case
end function
# llh_setdata()

######################################################################
function llh_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "src_key"
        display by name p_xinfor.src_key attribute(reverse)
      when scr_fld = "src_desc"
        display by name p_xinfor.src_desc attribute(reverse)
      when scr_fld = "src_num_desc"
        display by name p_xinfor.src_num_desc attribute(reverse)
      when scr_fld = "src_char_desc"
        display by name p_xinfor.src_char_desc attribute(reverse)
      when scr_fld = "src_acct_no"
        display by name p_xinfor.src_acct_no attribute(reverse)
    end case
end function
# llh_high()

######################################################################
function llh_display()
######################################################################
# This function displays all p_* variables onto the screen
#
    display p_xinfor.* to allc.*  attribute(red)
end function
# llh_display()

######################################################################
function llh_p_prep()
######################################################################
# This function creates the p_* record from the m_* record
#
    let p_xinfor.src_key = m_xinfor.src_key
    let p_xinfor.src_desc = m_xinfor.src_desc
    let p_xinfor.src_num_desc = m_xinfor.src_num_desc
    let p_xinfor.src_char_desc = m_xinfor.src_char_desc
    let p_xinfor.src_acct_no = m_xinfor.src_acct_no

    # Perform lookups
    if llh_lookup("acct_no_look",false) then end if

    let sqlca.sqlcode = 0
    if p_src_type = "W"
    then
      if alt_lookup("dept_look",false) then end if
    end if
    if p_src_type = "L"
    then
      if alt_lookup("comm_look",false) then end if
    end if
    let sqlca.sqlcode = 0


end function
# llh_p_prep()

######################################################################
function llh_m_prep()
######################################################################
# This function creates the m_* record from the p_* record
#
    let m_xinfor.src_key = p_xinfor.src_key
    let m_xinfor.src_desc = p_xinfor.src_desc
    let m_xinfor.src_num_desc = p_xinfor.src_num_desc
    let m_xinfor.src_char_desc = p_xinfor.src_char_desc
    let m_xinfor.src_acct_no = p_xinfor.src_acct_no
    if m_xinfor.src_type is null
    then
      let m_xinfor.src_type = p_src_type
    end if


end function
# llh_m_prep()

######################################################################
function llh_read(ring_rowid)
######################################################################
# This function loads the header p_* record elements.
# It first loads the m_* record from disk, then it
# calls the p_prep routine to fill the p_* record elements.
#
    define
        ring_rowid  integer     # rowid of record to read

    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch = "select * from stxinfor where rowid = ?"
        prepare read_specs from scratch
        declare h_cursor cursor for read_specs
    end if

    # Read the header record
    open h_cursor using ring_rowid
    fetch h_cursor into m_xinfor.*
    if sqlca.sqlcode != 0 then return end if
    let q_xinfor.row_id = ring_rowid
    close h_cursor
    call llh_p_prep()
end function
# llh_read()

######################################################################
function llh_delete(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    delete from stxinfor
        where rowid = ring_rowid
end function
# llh_delete()

######################################################################
function llh_add()
######################################################################
# This function inserts data into the header table.
#
    define
        new_rowid integer  # Rowid after insert
    # Insert the data
    insert into stxinfor values(m_xinfor.*)
    let new_rowid = sqlca.sqlerrd[6]

    # Reset rowid
    let sqlca.sqlerrd[6] = new_rowid

end function
# llh_add()

######################################################################
function llh_update(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    update stxinfor set stxinfor.* = m_xinfor.*
      where rowid = ring_rowid
end function
# llh_update()

######################################################################
function llh_lookup(tbl_name,mustfind)
# returning false if notfound
######################################################################
# This is the lookup function.  It is passed a table to lookup, and
# based on that, it looks up the key and fills any variables for
# that table.  The lookup function will produce the error if it is
# called with mustfind=true, and a notfound situation exists.
#
    define
        tbl_name char(18),  # Lookup table name
        stat integer,       # Value of sqlca.sqlcode after lookup
        mustfind smallint   # (boolean) produce an error if notfound?

    # Prepare cursors upon first call to this function
    if lookup_prep is null
    then
        let lookup_prep = "Y"
        let scratch = "select acct_desc ",
          "from stxchrtr where ",
          " stxchrtr.acct_no = ?"
        prepare str_acct_no_look from scratch
        declare cur_acct_no_look cursor for str_acct_no_look
    end if

    # Perform the lookup
    case
      when tbl_name = "acct_no_look"
        open cur_acct_no_look using p_xinfor.src_acct_no
        fetch cur_acct_no_look into p_xinfor.acct_desc
        let stat = sqlca.sqlcode
        close cur_acct_no_look
    end case

    # No mustfind if this_data is empty
    if length(this_data) = 0 then let mustfind = false end if

    # Send notfound message to user (if requested)
    let sqlca.sqlcode = stat
    if mustfind and stat = notfound
      then call scr_error("lookup=notfound",tbl_name)
    end if
    let sqlca.sqlcode = 0

    # Return false if lookup failed and empty into fields
    if stat = notfound
    then
        case
          when tbl_name = "acct_no_look"
            initialize p_xinfor.acct_desc to null
        end case
        return false
    else
        return true
    end if
end function
# llh_lookup()


######################################################################
function mlh_construct()
# returning criteria
######################################################################
#
    construct scratch
      on
        stxinfor.src_key, stxinfor.src_desc
      from
        allc.src_key, allc.src_desc

    let scratch = scratch clipped, " and ", p_sel_filter clipped
    return scratch[1,512]
end function
# mlh_construct

#####################################################################
function check_dup2()
# checks for duplicate key where key consists of two fields
# returns true if a duplicate is found; otherwise returns false
#####################################################################
define
    p_rowid smallint,
    p_count smallint

    if dup_prep is null
    then
        let dup_prep = "Y"
        let scratch = "select count(*) from stxinfor ",
                      "where src_type = ? and ",
                      "src_key = ? and rowid != ?"
        prepare dup_stmt from scratch
        declare dup_cur cursor for dup_stmt
    end if

    let p_rowid = get_scrlib("curs_rowid")
    open dup_cur using p_src_type, p_xinfor.src_key, p_rowid
    fetch dup_cur into p_count
    close dup_cur
    if p_count > 0 then return true end if
    return false
end function
# check_dup2()
