########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/lib.4gs/gl_sumrp.4gl  1.5  Delta: 2/11/91
########################################################################

globals "globals.4gl"

define
    table_open char(1),        # Is the temp table open?
    last_doc_no char(10)       # Last doc_no used for gl_sumget()

######################################################################
function gl_sumput(doc_no, acct_period, acct_year, acct_no,
  department, amount, debit_credit)
######################################################################
# This function adds this detail record to the temp table.
# The entire record must be passed to this function.
# If the temp table isn't open, it will create it.
#
    define
        doc_no       integer,     # Document number
        acct_period  char(2),     # Period number
        acct_year    char(4),     # Period year
        acct_no      integer,     # Account number
        department   char(3),     # Department
        amount       decimal(12), # Amount
        debit_credit char(1),     # D/C
        period  char(8)           # Period in "MM/YYYY" format

    # Trap fatal errors
    whenever error call error_handler

    # Create table?
    if table_open is null
    then
        let scratch =
            "create temp table sumtab( ",
               "doc_no       integer, ",
               "period       char(8), ",
               "acct_no      integer, ",
               "department   char(3), ",
               "amount       decimal(12), ",
               "debit_credit char(1))"
         if is_4_0()
         then
             let scratch = scratch clipped, " with no log"
         end if
         prepare create_prep from scratch
         execute create_prep
         let scratch = "insert into sumtab values (?, ?, ?, ?, ?, ?)"
         prepare ins_prep from scratch
         let table_open = "Y"
    end if

    # Place values into the temp table
    let period = acct_period, "/", acct_year
    if acct_period is null
    then
        let period = "*FUTURE*"
    end if
    execute ins_prep
      using doc_no, period, acct_no, department, amount, debit_credit
      
end function
# gl_sumput()

######################################################################
function gl_sumget(doc_no)
# returning true if there is any data to get
######################################################################
# This function serves as the data interface to the G/L summary data.
# You pass it a doc_no (or 0 for all documents), and it builds a cursor
# and returns the next row of data.
# It is designed to be used in a control loop.
# It can be used to get the summary of a document (via doc_no), or
# the summary of the entire table (via doc_no = 0).
# For an example of how to use it, see the gl_sumrpt() function below.
#
    define
        doc_no char(10),       # Doc_no to use (or 0 for all doc_no's)
        period char(8),        # Accounting period ("MM/YYYY")
        acct_no integer,       # Account number
        department char(3),    # Department
        amount decimal(12),    # Summary Amount
        debit_credit char(1)   # D/C

    # Open the cursor again?
    if doc_no = last_doc_no
    then else
        # Build the select string
        let scratch = "select period, acct_no, department, ", 
          "sum(amount), debit_credit from sumtab"
        if doc_no != 0
          then let scratch = scratch clipped, " where doc_no = ", doc_no
        end if
        let scratch = scratch clipped, " group by period, acct_no, ",
          "department, debit_credit order by period, acct_no, ",
          "department, debit_credit desc"
        prepare sum_specs from scratch
        declare sum_curs cursor for sum_specs
        open sum_curs
        let last_doc_no = doc_no
    end if

    # Fetch the next row
    fetch sum_curs into period, acct_no, department,
          amount, debit_credit

    # Return false if no more to find
    if sqlca.sqlcode = notfound
    then
        # Reset the last sumfilter
        let last_doc_no = null
        return false
    end if

    # Return the data in vararg scratch
    call put_vararg(period)
    call put_vararg(acct_no)
    call put_vararg(department)
    call put_vararg(amount)
    call put_vararg(debit_credit)
    return true

end function
# gl_sumget()

######################################################################
function gl_sumrpt()
######################################################################
# This function summarizes all data from the temp table and produces
# a report onto disk called "sumrpt.out" in this directory. 
#
    define
        account char(15),     # Full Length account number
        acct_desc char(30),    # Account description
        period char(8),        # Accounting period (MM/YYYY)
        acct_period char(2),   # Accounting period number (MM)
        acct_year char(4),     # Accounting period year (YYYY)
        acct_no integer,       # Account number
        department char(3),    # Department
        amt decimal(12),       # Summary Amount
        db_cr char(1),         # "C"redit or "D"ebit
        db_amt decimal(12),    # Summary Amount (debit)
        cr_amt decimal(12)     # Summary Amount (credit)

    # if 'no elements selected' the temp table will not exist
    if table_open is null then return end if

    start report glsumrpt to "sumrpt.out"
    while gl_sumget(0)

        # Get the data from gl_sumget()
        let period = get_vararg()
        let acct_no = get_vararg()
        let department = get_vararg()
        let amt = get_vararg()
        let db_cr = get_vararg()

        # Set the account and db_amt/cr_amt
        let account = acct_no using "<<<<<<<<<", "-", department
        let db_amt = null
        let cr_amt = null
        if db_cr = "D"
          then let db_amt = amt
          else let cr_amt = amt
        end if
        call al_chrtn(acct_no) returning acct_desc, db_cr

        output to report
          glsumrpt(period, account, acct_desc, db_amt, cr_amt)

    end while
    finish report glsumrpt

end function
# gl_sumrpt()

#######################################################################
report glsumrpt(rpt)
#######################################################################
# This is the image of the glsumrpt report
#
    define
        rpt record
            period char(8),        # Accounting period
            account char(15),      # Account number
            acct_desc char(30),    # Account description
            db_amt decimal(12),    # Debit amount
            cr_amt decimal(12)     # Credit amount
        end record

    output
        left margin 0
        right margin 0
        top margin 0
        bottom margin 0
        page length 1

    order external by rpt.period, rpt.account

    format

        before group of rpt.period
            print
            print
                column  27, "G/L Account Summary:",
                column  48, rpt.period
            print
            print
                column   4, "Account",
                column  19, "Description",
                column  57, "Debits",
                column  72, "Credits"
            print
                column   3, "--------------------------------------",
                column  41, "--------------------------------------"

        after group of rpt.account
            print
                column   4, rpt.account,
                column  19, rpt.acct_desc,
                column  49, group sum(rpt.db_amt) using "(((,(((,(((.&&)",
                column  65, group sum(rpt.cr_amt) using "(((,(((,(((.&&)"

        after group of rpt.period
            print column  49, "==============================="
            print
                column  49, group sum(rpt.db_amt) using "(((,(((,(((.&&)",
                column  65, group sum(rpt.cr_amt) using "(((,(((,(((.&&)"

# Report alignment mask:
#        1         2         3         4         5         6         7         8
#2345678901234567890123456789012345678901234567890123456789012345678901234567890
#
#                         G/L Account Summary: 08/1990
#
#  Account        Description                            Debit          Credit
# -----------------------------------------------------------------------------
#  ACCT/DEPT      XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX(((,(((,(((.&&) (((,(((,(((.&&)
#  1000-200       Cash in Bank - Bank of America                      4,210.22
#  1000-300       Cash in Bank - Bank of America     10,966.99
#                                               ===============================
#                                               (((,(((,(((.&&) (((,(((,(((.&&)

end report
# glsumrpt()
