########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/lib.4gs/init.4gl  1.9  Delta: 2/12/91
########################################################################

database standard

globals "globals.4gl"

############################################################
function init()
############################################################
# This function replaces the old st_init as of FourGen
# Accounting 3.5 modules.
#
# This function serves the purpose of initializing the
# accounting toolkit. It performs all of the required logic
# from $fg/lib/scr.4gs/init() plus local accounting
# initialization. The global and local were consolidated
# here for easier maintenance of entwined database
# manipulation and selection.
#
#   - general initialization
#   - sets default global library values
#   - adds library tables to the database by running fg_tables()
#   - handles processing for multi-company
#   - handles turning on/off database
#     transaction processing.
#
    define
        ini_mod char(8),     # module name
        ini_prog char(8),    # program name
        ini_trx smallint,    # use transactions ? (boolean true/false)
        compid char(8),      # company identification
        dbname char(14),     # database name for that company id
        logpath char(64),    # pathname of that database's logfile
        tmp_str char(64),    # scratch string
        n smallint           # working number

    call startlog("errlog")

    # Trap fatal errors
    whenever error call error_handler

    # General initialization
    options input wrap
    options prompt line first + 1
    options form line first + 3
    options help key control-q
    call logo()

    # Set the language variable
    for n = 1 to num_args() - 1
        if arg_val(n) = "-l"
        then
            let tmp_str = arg_val(n + 1)
        end if
    end for
	if tmp_str is null or tmp_str = " "
    then
      let tmp_str = "ENG"
    end if
    call put_scrlib("language",tmp_str)

    # Default some values
    if get_scrlib("auto_udf") is null
      then call put_scrlib("auto_udf","Y")
    end if
    if get_scrlib("auto_note") is null
      then call put_scrlib("auto_note","N")
    end if

    # Get module and program names
    call get_mod() returning ini_mod, ini_prog

    # Default to the standard database
    let compid = null
    let logpath = null

    # Open the compile database
    let dbname = get_scrlib("dbname")
    if dbname is null or dbname = " "
    then
        let dbname = "standard"
    end if
    database dbname

    # Parse the command line for "-c $company" or "-d dbname"
    # where company is the company id that is used as a key into 
    # stxcompr to find the database name, and the logfile path.
    for n = 1 to num_args() - 1
        if arg_val(n) = "-c" or arg_val(n) = "-d"
        then
            let compid = arg_val(n+1)
        end if
    end for

    # Try to find the database name in stxcompr in the standard
    # database
    if compid is not null
    then
        select db_name, logfile into  dbname, logpath
            from stxcompr where comp_id = compid
        if sqlca.sqlcode = notfound
        then
            # Try opening the database to see if it exists
            whenever error continue
            database compid
            whenever error call error_handler
            if sqlca.sqlcode = 0
              then
                  let dbname = compid
              else
                  call errorlog (sqlca.sqlcode)
                  let sqlca.sqlcode = 0
                  call fg_error("lib_all","st_init", 1)
                      # Specified company not found - using standard
            end if
        end if
    end if

    # Make sure dbname is not null
    if dbname is null or dbname = " "
    then
        let dbname = "standard"
    end if

    # Default the logpath
    if logpath is null or logpath = " "
    then
        let logpath = "../../data/", dbname[1,10] clipped, ".log"
    end if

    # Open the chosen database
    database dbname
    call put_scrlib("dbname",dbname)

    # Set isolation level for online/turbo database
    if is_turbo()
    then
        set isolation to dirty read
    end if

    # Check to see if the most recent schema has been installed
    # onto this database.
    # Latest change: addition of the cgstrigr table.
# not yet...
#   select count(*) into n from systables where tabname = "cgstrigr"
#   if n = 0
#   then
#       # Install the tables
#       call fg_tables()
#   end if

    # To keep nrows in systables as current as possible, update
    # statistics every time a posting program is run.
    if ini_prog[1,2] = "p_"
    then
        whenever error continue
        update statistics
        whenever error call error_handler
    end if

    # If the current sql engine is the old (.03) standard engine,
    # then check if logging should be started.
    if log_change_ok()
    then
        select use_trx into ini_trx from stxprogr where
            module = ini_mod and progname = ini_prog
        if sqlca.sqlcode = notfound
        then
            # Set default trx_processing to true if post type process...
            let sqlca.sqlcode = 0
            if ini_prog[1,2] = "p_"
            then
                let ini_trx = true
            else
                let ini_trx = false
            end if
        end if

        if ini_trx = true
          then call add_log(dbname,logpath)
          else call chk_log(dbname,logpath)
        end if
    end if

    return
end function
# init()

############################################################
function add_log(dbname,logpath)
############################################################
# Add a record "syslog" into systables.
# Close and reopen the database for transaction logging.
# Deletes the record "syslog"
#
    define
        log_rowid integer,
        dbname char(14),
        logpath char(64)

    select rowid into log_rowid from systables where tabname = "syslog"
    if sqlca.sqlcode = notfound
    then
        insert into systables
            (tabname,owner,dirpath,tabid,rowsize,ncols,
            nindexes,nrows,created,version,tabtype)
        values
            ("syslog","informix",logpath,
              0,0,1,0,0,today,0,"L")
        database dbname
        delete from systables where tabname = "syslog"
    end if

end function
# add_log()

############################################################
function chk_log(dbname,logpath)
############################################################
# Check the table systables to see if transaction is set on.
# If transaction is on, then it sleeps until the transaction
# record is removed.
#
    define
        sys_rowid integer,
        dbname char(14),
        logpath char(64),
        cnt, i, j smallint

        select rowid into sys_rowid from systables
            where tabname = "syslog"

        if sqlca.sqlcode = notfound   #database was open without transaction
        then
            let sqlca.sqlcode = 0
        else                 # wait for database to open without trx
            open window wait_sys at 2,4 with 2 rows, 60 columns
                attribute(white,border)
            display 
                " Waiting for system table to become available..." at 1,1
            let cnt = 5
            while cnt > 0
                # let the user know we're doing something
                # even though we aren't
                for i = 1 to 5
                    display " ", cnt, " " at 2, 10 attribute (reverse)
                    let j = 10 + (i * 4)
                    display " * " at 2, j
                    sleep 1
                end for
                display "" at 2, 1

                select rowid into sys_rowid from systables
                    where tabname = "syslog"
                if sqlca.sqlcode = 0
                then
                    let cnt = cnt - 1
                    continue while
                else
                    let sqlca.sqlcode = 0
                    exit while
                end if
            end while
            if cnt = 0
            then
                # problem with syslog record.  it should never be there
                # for this long.  the only time the syslog record should
                # be there is when you run a process with transactions,
                # and in that case, the record is inserted only for the
                # time it takes to close the database, re-open it (with
                # transactions), then delete that syslog record.
                # were going to have to 86 that syslog record...

                delete from systables where rowid = sys_rowid
            end if

            close window wait_sys
            database dbname
        end if
end function
# chk_log()


########################################################################
function log_change_ok()
# determines if it is possible to change transaction logging
########################################################################
# returns true if ok to change transaction logging, false if not.
# also ensures logging is on for 4.0 standard engine

    # check for "dirpath". if not found, then turbo/on-line
    select rowid from syscolumns where tabid = 1 and colname = "dirpath"
    if sqlca.sqlcode = notfound
    then
        let sqlca.sqlcode = 0
        return false     # because we can't toggle turbo/on-line
                         # transaction processing
    end if

    # check for "text".  if found, then pre-4.0, so return true
    select rowid from syscolumns where tabid = 6 and colname = "text"
    if sqlca.sqlcode = 0
    then
        return true     # yes, we can toggle transactions
    end if

    # if here, must be 4.0
    # check for the existance of a log.  if not there, explain the
    # problem and then exit the program.
    select rowid from systables where tabname = "syslog"
    if sqlca.sqlcode = notfound
    then
        let sqlca.sqlcode = 0
        call fg_error("lib_all","st_init",2) # must be a log for inx 4.0
        exit program (1)
    end if

    # else the log exists for inx 4.0, return false
    return false
end function
# log_change_ok()
