########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/lib.4gs/xfer_udf.4gl  1.3  Delta: 2/9/91
########################################################################

database standard

#######################################################################
function xfer_udf(old_fname, new_fname, old_key, new_key, xfer_type)
#######################################################################
# This function will copy, move, or delete user define fields and notes
# from one key to another.  this is frequenty used by posting 
# programs to transfer the link from the entry table to the the 
# transaction table.
#
# xfer_type: "M" - move   - copy udf's from old key to new key
#                         - delete udf's using old key
#            "C" - move   - copy udf's from old key to new key
#            "D" - delete - delete udf's with old key and old_fname
#
    define
        old_fname like stxaddld.filename,
        new_fname like stxaddld.filename,
        old_key like stxaddld.record_key,
        new_key like stxaddld.record_key,
        xfer_type char(1)

    whenever error call error_handler

    case 
        when xfer_type = "M"
            call move_notes(old_fname, new_fname, old_key, new_key)
            call move_udf(old_fname, new_fname, old_key, new_key)
        when xfer_type = "C"
            call copy_notes(old_fname, new_fname, old_key, new_key)
            call copy_udf(old_fname, new_fname, old_key, new_key)
        when xfer_type = "D"
            call delete_notes(old_fname, old_key)
            call delete_udf(old_fname, old_key)
        otherwise
            exit case
    end case

end function
# xfer_udf

########################################################################
function move_notes(old_fname, new_fname, old_key, new_key)
########################################################################
#
# this function will move user defined notes to the new key
#
    define
        old_fname like stxaddld.filename,
        new_fname like stxaddld.filename,
        old_key like stxaddld.record_key,
        new_key like stxaddld.record_key

    update stxnoted set (filename, record_key) = (new_fname, new_key)
        where filename = old_fname and record_key = old_key
    
end function
# move_notes

########################################################################
function move_udf(old_fname, new_fname, old_key, new_key)
########################################################################
#
# this function will move a set of user define fields to it's new key
#
    define
        old_fname like stxaddld.filename,
        new_fname like stxaddld.filename,
        old_key like stxaddld.record_key,
        new_key like stxaddld.record_key

    update stxaddlr set (filename) = (new_fname)
        where filename = old_fname

    update stxaddld set (filename, record_key) = (new_fname, new_key)
        where filename = old_fname and record_key = old_key

end function
# move_udf

########################################################################
function copy_notes(old_fname, new_fname, old_key, new_key)
########################################################################
#
# this function will copy a set of user define notes to it's new key
#
    define
        old_fname like stxaddld.filename,
        new_fname like stxaddld.filename,
        old_key like stxaddld.record_key,
        new_key like stxaddld.record_key,
        sql_str char(256),
        quote char(1)

    let quote = "\""

    let sql_str = 
        "select * from stxnoted ",
            "where filename = ", quote, old_fname, quote,
            " and record_key = ", quote, old_key, quote,
                " into temp temp_notes"

    if is_4_0()
    then
        let sql_str = sql_str clipped, " with no log"
    end if

    prepare create_tmp from sql_str
    execute create_tmp

    update temp_notes set (filename, record_key) = (new_fname, new_key)

    insert into stxnoted select * from temp_notes

    drop table temp_notes

end function
# copy_notes

########################################################################
function copy_udf(old_fname, new_fname, old_key, new_key)
########################################################################
#
# this function will copy a set of user define fields to it's new key
#
    define
        old_fname like stxaddld.filename,
        new_fname like stxaddld.filename,
        old_key like stxaddld.record_key,
        new_key like stxaddld.record_key,
        sql_str char(256),
        quote char(1)

    let quote = "\""

    let sql_str = 
        "select * from stxaddlr ",
            "where filename = ", quote, old_fname, quote,
                " into temp temp_header"

    if is_4_0()
    then
        let sql_str = sql_str clipped, " with no log"
    end if
 
    prepare create_header from sql_str
    execute create_header

    update temp_header set (filename) = (new_fname)

    insert into stxaddlr select * from temp_header

    drop table temp_header

    let sql_str = 
        "select * from stxaddld ",
            "where filename = ", quote, old_fname, quote,
            " and record_key = ", quote, old_key, quote,
                " into temp temp_detail"

    if is_4_0()
    then
        let sql_str = sql_str clipped, " with no log"
    end if

    prepare create_detail from sql_str
    execute create_detail

    update temp_detail set (filename, record_key) = (new_fname, new_key)

    insert into stxaddld select * from temp_detail

    drop table temp_detail

end function
# copy_udf

########################################################################
function delete_notes(old_fname, old_key)
########################################################################
#
#
#
    define
        old_fname like stxaddld.filename,
        old_key like stxaddld.record_key

    delete from stxnoted 
        where filename = old_fname and record_key = old_key

end function
# delete_notes

########################################################################
function delete_udf(old_fname, old_key)
########################################################################
#
#
#
    define
        old_fname like stxaddld.filename,
        old_key like stxaddld.record_key

    delete from stxnoted 
        where filename = old_fname and record_key = old_key

end function
# delete_udf
