########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../all.4gm/o_gldetl.4gs/midlevel.4gl  1.8  Delta: 1/31/91
########################################################################

globals "globals.4gl"

#######################################################################
function ml_join()
#######################################################################
#
    define
        start_date date,
        end_date date,
        start_period like stxperdr.period,
        start_year like stxperdr.period_year,
        end_period like stxperdr.period,
        end_year like stxperdr.period_year,
        query_string char(80),
        quote char(1)

    # Trap fatal errors
    whenever error call error_handler

    let quote = "\""
    if ct.prcname = "G/L Activity Detail"
    then
        call acct_per("distd")
            returning start_period, start_year,
                    end_period, end_year, query_string

        # a syntax error will occur if the query_string being returned
        # by the acct_per() function is not prepended with the
        # following.  otherwise, if query_string is null, this should
        # not be done.
        if query_string is not null
        then
            let query_string = " and ", query_string clipped
        end if

        let rpt.head1 = null

        select stxperdr.start_date
          into s_date
            from stxperdr
              where stxperdr.period = start_period and
                    stxperdr.period_year = start_year

        select stxperdr.end_date 
          into e_date
            from stxperdr
              where stxperdr.period = end_period and
                    stxperdr.period_year = end_year

        let rpt.head2 = "For the Period of: ", start_period, "/",
                        start_year, " - ",end_period,"/",end_year,
                        "     ", s_date, " to ", e_date

        let p_sel_join = "stxtranr.doc_no = stgtranr.doc_no and",
            " stxtranr.orig_journal = stgtranr.orig_journal and",
            " stgactvd.doc_no = stgtranr.doc_no and",
            " stgactvd.orig_journal = stgtranr.orig_journal and",
            " (stgtranr.acct_year > ", quote, start_year, quote,
            " or (stgtranr.acct_period >= ", quote, start_period, quote,
            " and stgtranr.acct_year = ", quote, start_year, quote,
            " )) and (stgtranr.acct_year < ", quote, end_year, quote,
            " or (stgtranr.acct_period <= ", quote, end_period, quote,
            " and stgtranr.acct_year = ", quote, end_year, quote,
            " ))",query_string clipped
    else
        call dat_rnge() returning start_date, end_date
        let rpt.head1 = ct.prcname
        let rpt.head2 = "Between ", start_date," and ",end_date
        let p_sel_join = "stxtranr.doc_no = stgtranr.doc_no and",
            " stxtranr.orig_journal = stgtranr.orig_journal and",
            " stgactvd.doc_no = stgtranr.doc_no and",
            " stgactvd.orig_journal = stgtranr.orig_journal and",
            " stxtranr.doc_date >= ", quote, start_date, quote,
            " and stxtranr.doc_date <= ", quote, end_date,quote
    end if

end function
# ml_join()

#######################################################################
function ml_filter()
#######################################################################
#
    let ct.sel_filter = "1 = 1"
    return
end function
# ml_filter()

#######################################################################
function ml_order()
#######################################################################
#
    let ct.sel_order = "stgactvd.acct_no, stgactvd.department,",
                       "stxtranr.doc_date, stxtranr.orig_journal,",
                       "stxtranr.doc_no"
    return
end function
# ml_order()

#######################################################################
function ml_getcount()
#######################################################################
#
    let ct.sel_stmt =
        "select count(*) ",
        "from stgtranr, stxtranr, stgactvd ",
        "where (", p_sel_join clipped,
        ") and (", ct.sel_filter clipped, ")"
    prepare get_count from ct.sel_stmt
    declare count_cursor cursor for get_count
    open count_cursor
    fetch count_cursor into ct.num_rows
    close count_cursor
    return
end function
# ml_getcount()

#######################################################################
function ml_define_cur()
#######################################################################
#
    let ct.sel_stmt =
        "select ",
            "stgactvd.amount, ",
            "stgactvd.debit_credit, ",
            "stgtranr.acct_period, ",
            "stgtranr.acct_year, ",
            "stgactvd.acct_no, ",
            "stgactvd.department, ",
            "stxtranr.orig_journal, ",
            "stxtranr.doc_no, ",
            "stxtranr.doc_date, ",
            "stxtranr.ref_code, ",
            "stxtranr.doc_desc ",
        "from stgtranr, stxtranr, stgactvd ",
        "where (", p_sel_join clipped,
        ") and (", ct.sel_filter clipped, ")"
    if ct.sel_order is not null
    then
        let ct.sel_stmt = ct.sel_stmt clipped,
        " order by ", ct.sel_order clipped
    end if
    prepare get_curs from ct.sel_stmt
    declare rpt_cursor cursor for get_curs
    open rpt_cursor
    return
end function
# ml_define_cur()

#######################################################################
function ml_fetch()
#######################################################################
#
    fetch rpt_cursor into curs_next.*
    return
end function
# ml_fetch()

#######################################################################
function ml_curs_null()
#######################################################################
#  this function sets all of the elements in the curs
#  record to null
#
    initialize curs.* to null
end function
# ml_curs_null()

#######################################################################
function ml_next_null()
#######################################################################
#  this function sets all of the elements in the curs_next
#  record to null
#
    initialize curs_next.* to null
end function
# ml_next_null()

#######################################################################
function ml_curs_prep()
#######################################################################
#  this function transfers data from the curs_next record to the
#  curs record.
#
    let curs.amount = curs_next.amount
    let curs.debit_credit = curs_next.debit_credit
    let curs.acct_period = curs_next.acct_period
    let curs.acct_year = curs_next.acct_year
    let curs.acct_no = curs_next.acct_no
    let curs.department = curs_next.department
    let curs.orig_journal = curs_next.orig_journal
    let curs.doc_no = curs_next.doc_no
    let curs.doc_date = curs_next.doc_date
    let curs.ref_code = curs_next.ref_code
    let curs.doc_desc = curs_next.doc_desc
end function
# ml_curs_prep()

#######################################################################
function ml_before_break()
#######################################################################
#  this function calls before_group() when appropriate.
#
    define
        no_break    smallint    # set to false if control break
    let no_break = true
    if no_break and curs.acct_no = curs_next.acct_no
    then else
        if no_break then call ml_curs_prep() end if
        call before_group("acct_no")
        let no_break = false
    end if
    if no_break and curs.department = curs_next.department
    then else
        if no_break then call ml_curs_prep() end if
        call before_group("department")
        let no_break = false
    end if
    if no_break and curs.orig_journal = curs_next.orig_journal
    then else
        if no_break then call ml_curs_prep() end if
        call before_group("orig_journal")
        let no_break = false
    end if
    if no_break and curs.doc_date = curs_next.doc_date
    then else
        if no_break then call ml_curs_prep() end if
        call before_group("doc_date")
        let no_break = false
    end if
end function
# ml_before_break()

#######################################################################
function ml_after_break()
#######################################################################
#  this function calls after_group() when appropriate.
#
    define
        no_break    smallint    # set to false if control break
    let no_break = true
    if no_break and curs.acct_no = curs_next.acct_no
    then else
        call after_group("acct_no")
        let no_break = false
    end if
    if no_break and curs.department = curs_next.department
    then else
        call after_group("department")
        let no_break = false
    end if
    if no_break and curs.orig_journal = curs_next.orig_journal
    then else
        call after_group("orig_journal")
        let no_break = false
    end if
    if no_break and curs.doc_date = curs_next.doc_date
    then else
        call after_group("doc_date")
        let no_break = false
    end if
end function
# ml_after_break()

#######################################################################
function ml_output()
#######################################################################
#
    output to report report1 (rpt.*)
    return
end function
# ml_output()

#######################################################################
function ml_defaults()
#######################################################################
#  this function sets the ct.* record values
#
    let ct.prcname = "G/L Activity Detail"
    let ct.rtmargn = "FourGen Modifiable Accounting"
    let ct.destin = "report.out"
    let ct.quiet = "10"
    let rpt.co_name = al_compn()
end function
# ml_defaults()

#######################################################################
function acct_null(scratch)
#######################################################################
#  this funciton acts as a holding place until the acct_per function
#  can be called (after g/l is installed)
#
    define
        scratch char(10)

    return "", "", "", "", ""
end function
# ml_defaults()
